@@ -30,6 +30,7 @@ import com.sun.electric.database.topology.NodeInst;
 import com.sun.electric.database.topology.PortInst;
 import com.sun.electric.database.variable.EditWindow0;
 import com.sun.electric.database.variable.ElectricObject;
+import com.sun.electric.database.variable.TextDescriptor;
 import com.sun.electric.technology.Layer;
 import com.sun.electric.tool.Job;
 
@@ -38,10 +39,15 @@ import java.awt.Shape;
 import java.awt.font.GlyphVector;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Area;
+import java.awt.geom.Line2D;
 import java.awt.geom.PathIterator;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Comparator;
+import java.util.List;
+import java.util.Stack;
 
 /**
  * The Poly class describes an extended set of points
@@ -50,6 +56,7 @@ import java.util.*;
  */
 public class PolyBase implements Shape, PolyNodeMerge
 {
+	private static final boolean ALLOWTINYPOLYGONS = false;
 
 	/** the style (outline, text, lines, etc.) */			private Poly.Type style;
 	/** the points */										protected Point2D points[];
@@ -61,8 +68,9 @@ public class PolyBase implements Shape, PolyNodeMerge
     /** represents X axis */                                public static final int X = 0;
     /** represents Y axis */                                public static final int Y = 1;
     /** represents Z axis */                                public static final int Z = 2;
+    /** represents on the plane XY */                       public static final int XY = 4;
 
-	/**
+    /**
 	 * The constructor creates a new Poly given an array of points.
 	 * @param points the array of coordinates.
 	 */
@@ -141,6 +149,18 @@ public class PolyBase implements Shape, PolyNodeMerge
 		this.pp = null;
 	}
 
+    /**
+     * Convert to useful string
+     * @return a string describing this object
+     */
+    public String toString() {
+        StringBuffer buf = new StringBuffer();
+        if (layer != null) buf.append(layer.getName()+": ");
+        for (Point2D p : points) buf.append("("+p.getX()+", "+p.getY()+"), ");
+        if (style != null) buf.append(style.toString());
+        return buf.toString();
+    }
+
 	/**
 	 * Method to return the style associated with this Poly.
 	 * The style controls how the points are interpreted (FILLED, CIRCLE, etc.)
@@ -366,7 +386,7 @@ public class PolyBase implements Shape, PolyNodeMerge
         Rectangle2D box = getBounds2D();
 
         // The point is outside the bounding box of the polygon
-        if (!DBMath.pointInsideRect(pt, box))
+        if (!DBMath.pointInRect(pt, box))  //pointInsideRect. It could be at the edges
             return false;
 
         int count = 0;
@@ -767,14 +787,64 @@ public class PolyBase implements Shape, PolyNodeMerge
 		// can only rotate anchor when node is in a manhattan orientation
 		if ((nodeAngle%900) != 0) return origType;
 
+		// special case handling for left/right/top/bottom orientations
+		if (origType == Poly.Type.TEXTLEFT || origType == Poly.Type.TEXTRIGHT ||
+			origType == Poly.Type.TEXTTOP || origType == Poly.Type.TEXTBOT)
+		{
+			boolean flipAnchor = false;
+			if (ni.isMirroredAboutXAxis())
+			{
+				if (ni.isMirroredAboutYAxis())
+				{
+					// mirrored in both directions: flip only the 0-degree anchor
+					if (nodeAngle == 0) flipAnchor = true;
+				} else
+				{
+					// mirrored only U-D: flip conditionally
+					if (origType == Poly.Type.TEXTLEFT || origType == Poly.Type.TEXTRIGHT)
+					{
+						if (nodeAngle == 900 || nodeAngle == 1800) flipAnchor = true;
+					} else if (origType == Poly.Type.TEXTTOP || origType == Poly.Type.TEXTBOT)
+					{
+						if (nodeAngle == 0 || nodeAngle == 2700) flipAnchor = true;
+					}
+				}
+			} else
+			{
+				if (ni.isMirroredAboutYAxis())
+				{
+					// mirrored only L-R: flip  conditionally
+					if (origType == Poly.Type.TEXTLEFT || origType == Poly.Type.TEXTRIGHT)
+					{
+						if (nodeAngle == 0 || nodeAngle == 2700) flipAnchor = true;
+					} else if (origType == Poly.Type.TEXTTOP || origType == Poly.Type.TEXTBOT)
+					{
+						if (nodeAngle == 900 || nodeAngle == 1800) flipAnchor = true;
+					}
+				} else
+				{
+					// not mirrored: flip only 180-degree anchor
+					if (nodeAngle == 1800) flipAnchor = true;
+				}
+			}
+			if (flipAnchor)
+			{
+				if (origType == Poly.Type.TEXTLEFT) return Poly.Type.TEXTRIGHT;
+				if (origType == Poly.Type.TEXTRIGHT) return Poly.Type.TEXTLEFT;
+				if (origType == Poly.Type.TEXTTOP) return Poly.Type.TEXTBOT;
+				if (origType == Poly.Type.TEXTBOT) return Poly.Type.TEXTTOP;
+			}
+			return origType;
+		}
+
 		// determine angle of original style
 		int origAngle = origType.getTextAngle();
-		if (ni.isMirroredAboutXAxis() != ni.isMirroredAboutYAxis() && ((origAngle%1800) == 0 || (origAngle%1800) == 1350)) origAngle += 1800;
+		if (ni.isMirroredAboutXAxis() != ni.isMirroredAboutYAxis() && ((origAngle%1800) == 0 || (origAngle%1800) == 1350))
+			origAngle += 1800;
 
 		// determine change in angle because of node rotation
         Orientation orient = Orientation.fromJava(nodeAngle, ni.isMirroredAboutXAxis(), ni.isMirroredAboutYAxis());
         AffineTransform trans = orient.pureRotate();
-//		AffineTransform trans = NodeInst.pureRotate(nodeAngle, ni.isMirroredAboutXAxis(), ni.isMirroredAboutYAxis());
 		Point2D pt = new Point2D.Double(100, 0);
 		trans.transform(pt, pt);
 		int xAngle = GenMath.figureAngle(new Point2D.Double(0, 0), pt);
@@ -944,7 +1014,7 @@ public class PolyBase implements Shape, PolyNodeMerge
 				localStyle = Poly.Type.CLOSED;
 			} else
 			{
-				if (otherBounds.intersects(polyBounds)) return Double.MIN_VALUE;
+				if (DBMath.rectsIntersect(otherBounds, polyBounds)) return Double.MIN_VALUE;
 				return otherPt.distance(polyCenter);
 			}
 		}
@@ -968,7 +1038,7 @@ public class PolyBase implements Shape, PolyNodeMerge
 				return bestDist;
 			} else
 			{
-				if (otherBounds.intersects(polyBounds)) return Double.MIN_VALUE;
+				if (DBMath.rectsIntersect(otherBounds, polyBounds)) return Double.MIN_VALUE;
 				return otherPt.distance(polyCenter);
 			}
 		}
@@ -993,7 +1063,6 @@ public class PolyBase implements Shape, PolyNodeMerge
 			} else
 			{
 				if (DBMath.rectsIntersect(otherBounds,polyBounds)) return Double.MIN_VALUE;
-//				if (otherBounds.intersects(polyBounds)) return Double.MIN_VALUE;
 				return otherPt.distance(polyCenter);
 			}
 		}
@@ -1341,7 +1410,6 @@ public class PolyBase implements Shape, PolyNodeMerge
 	/**
 	 * Method to tell whether a rectangle intersects this Poly.
 	 * This method is a requirement of the Shape implementation.
-	 * THIS METHOD HAS NOT BEEN WRITTEN YET!!!
 	 * @param r the rectangle.
 	 * @return true if the rectangle intersects the Poly.
 	 */
@@ -1395,7 +1463,195 @@ public class PolyBase implements Shape, PolyNodeMerge
 		return false;
 	}
 
-	/**
+    /**
+     * Method to find the intersection area of this poly with another poly.
+     * Returns null if no intersection.
+     * @param polyOther the other poly
+     * @param overlappingEdges if non-null, this list will be filled with edges that overlap
+     * between the two polygons. This is useful if the polygons touch, as the intersection area will be
+     * empty.
+     * @return the intersection area of the two, or null if none
+     */
+    public List<PolyBase> getIntersection(PolyBase polyOther, List<Line2D> overlappingEdges)
+    {
+        // get overlapping edges if requested
+        if (overlappingEdges != null) {
+            List<Line2D> overlaps = getOverlappingEdges(polyOther);
+            overlappingEdges.addAll(overlaps);
+        }
+        // get intersected area
+        Area myArea = new Area(this);
+        Area otherArea = new Area(polyOther);
+        myArea.intersect(otherArea);
+
+        List<PolyBase> polys = getPointsInArea(myArea, layer, true, false);
+        if (polys == null) return null;
+        return polys;
+    }
+
+    /**
+     * Get the overlapping edges with other polygon. Returns an empty list if none.
+     * @param polyOther the other polygon
+     * @return a list of overlapping edges
+     */
+    public List<Line2D> getOverlappingEdges(PolyBase polyOther)
+    {
+        List<Line2D> overlappingSegs = new ArrayList<Line2D>();
+
+        // quit now if bounding boxes don't overlap
+        Rectangle2D thisBounds = getBounds2D();
+        Rectangle2D otherBounds = polyOther.getBounds2D();
+        if (thisBounds.getMaxX() < otherBounds.getMinX() ||
+            otherBounds.getMaxX() < thisBounds.getMinX() ||
+            thisBounds.getMaxY() < otherBounds.getMinY() ||
+            otherBounds.getMaxY() < thisBounds.getMinY()) return overlappingSegs;
+
+        // check if any line segments intersect, and add the intersection
+        // point to the list if points
+        List<Line2D> myLineSegs = getLineSegments();
+        List<Line2D> otherLineSegs = polyOther.getLineSegments();
+
+        for (Line2D line1 : myLineSegs)
+        {
+            for (Line2D line2 : otherLineSegs)
+            {
+                Line2D seg = getLineOverlap(line1, line2);
+                if (seg != null) overlappingSegs.add(seg);
+            }
+        }
+        // remove single points if they are redundant with line segments
+        List<Line2D> realLines = new ArrayList<Line2D>();
+        List<Line2D> points = new ArrayList<Line2D>();
+        for (Line2D seg : overlappingSegs)
+        {
+            if (seg.getP1().equals(seg.getP2()))
+                points.add(seg);
+            else
+                realLines.add(seg);
+        }
+        for (Line2D p : points)
+        {
+            boolean redundant = false;
+            for (Line2D seg : realLines)
+            {
+                if (p.getP1().equals(seg.getP1()) || p.getP1().equals(seg.getP2())) {
+                    redundant = true; break;
+                }
+            }
+            if (!redundant)
+                realLines.add(p);
+        }
+
+        return realLines;
+    }
+
+    // get the line segments of this polygon
+    private List<Line2D> getLineSegments()
+    {
+        List<Line2D> lineSegs = new ArrayList<Line2D>();
+        for(int i=0; i<points.length; i++)
+        {
+            Point2D p;
+            if (i == 0)
+            {
+                if (style == Poly.Type.OPENED || style == Poly.Type.OPENEDT1 ||
+                    style == Poly.Type.OPENEDT2 || style == Poly.Type.OPENEDT3 ||
+                    style == Poly.Type.VECTORS) continue;
+                p = points[points.length-1];
+            } else
+            {
+                p = points[i-1];
+            }
+            Point2D t = points[i];
+            if (style == Poly.Type.VECTORS && (i&1) != 0) i++;
+            if (p.getX() == t.getX() && p.getY() == t.getY()) continue;
+
+            // see if this line intersects a line on the other poly
+            lineSegs.add(new Line2D.Double(p, t));
+        }
+        return lineSegs;
+    }
+
+    /**
+     * Get the intersection point of two line segments, if any.
+     * This also returns null if the two lines are the same line,
+     * as all points on either lines are intersection points.
+     * @param line1 the first line segment
+     * @param line2 the second line segment
+     * @return the intersection point, or null if none
+     */
+    public static Point2D getLineSegmentIntersection(Line2D line1, Line2D line2)
+    {
+        if (!line1.intersectsLine(line2))
+            return null;
+        double [] co1 = getLineCoeffs(line1);
+        double [] co2 = getLineCoeffs(line2);
+        // det = A1*B2 - A2*B1
+        double det = co1[0]*co2[1] - co2[0]*co1[1];
+        // if det == 0, lines are parallel, but already checked by intersection check above
+        // if det == 0 and we got here, lines are the same line.
+        if (det == 0) return null;
+        // x = (B2*C1 - B1*C2)/det
+        double x = (co2[1]*co1[2] - co1[1]*co2[2])/det;
+        // y = (A1*C2 - A2*C1)/det
+        double y = (co1[0]*co2[2] - co2[0]*co1[2])/det;
+        if (x == -0.0) x = 0;
+        if (y == -0.0) y = 0;
+        return new Point2D.Double(x, y);
+    }
+
+    /**
+     * Get the line segment that is common to both lines. Returns null if none.
+     * @param line1 the first line segment
+     * @param line2 the second line segment
+     * @return the common line segment, or null if none
+     */
+    public static Line2D getLineOverlap(Line2D line1, Line2D line2)
+    {
+        if (!line1.intersectsLine(line2))
+            return null;
+        double [] co1 = getLineCoeffs(line1);
+        double [] co2 = getLineCoeffs(line2);
+        // det = A1*B2 - A2*B1
+        double det = co1[0]*co2[1] - co2[0]*co1[1];
+        // if det == 0, lines are parallel, but already checked by intersection check above
+        // if det == 0 and we got here, lines are the same line.
+        if (det != 0) return null;
+        double minX1 = Math.min(line1.getX1(), line1.getX2());
+        double minX2 = Math.min(line2.getX1(), line2.getX2());
+        double minX = Math.max(minX1, minX2);
+
+        double minY1 = Math.min(line1.getY1(), line1.getY2());
+        double minY2 = Math.min(line2.getY1(), line2.getY2());
+        double minY = Math.max(minY1, minY2);
+
+        double maxX1 = Math.max(line1.getX1(), line1.getX2());
+        double maxX2 = Math.max(line2.getX1(), line2.getX2());
+        double maxX = Math.min(maxX1, maxX2);
+
+        double maxY1 = Math.max(line1.getY1(), line1.getY2());
+        double maxY2 = Math.max(line2.getY1(), line2.getY2());
+        double maxY = Math.min(maxY1, maxY2);
+
+        Point2D p1 = new Point2D.Double(minX, minY);
+        Point2D p2 = new Point2D.Double(maxX, maxY);
+        return new Line2D.Double(p1, p2);
+    }
+
+    /**
+     * Get the coeffecients of the line of the form Ax + By = C.
+     * Can't use y = Ax + B because it does not allow x = A type equations.
+     * @param line the line
+     * @return an array of the values A,B,C
+     */
+    private static double [] getLineCoeffs(Line2D line) {
+        double A = line.getP2().getY() - line.getP1().getY();
+        double B = line.getP1().getX() - line.getP2().getX();
+        double C = A * line.getP1().getX() + B * line.getP1().getY();
+        return new double [] {A,B,C};
+    }
+
+    /**
 	 * Method to return true if the line segment from (px1,py1) to (tx1,ty1)
 	 * intersects any line in polygon "poly"
 	 */
@@ -1862,7 +2118,7 @@ public class PolyBase implements Shape, PolyNodeMerge
                     else if (Job.getDebug() && t.poly.contains(pn.getPoints()[0]))
                     {
                         assert(false);
-                        System.out.println(" Bad happen");
+                        System.out.println("Bad happen");
                     }
                 }
                 sons.add(t);
@@ -1919,16 +2175,25 @@ public class PolyBase implements Shape, PolyNodeMerge
 			int type = pIt.currentSegment(coords);
 			if (type == PathIterator.SEG_CLOSE)
 			{
-				Point2D [] points = new Point2D[pointList.size()];
-                System.arraycopy(pointList.toArray(), 0, points, 0, pointList.size());
-//                int i = 0;
-//				for(Point2D p : pointList)
-//					points[i++] = p;
-				PolyBase poly = new PolyBase(points);
-				poly.setLayer(layer);
-				poly.setStyle(Poly.Type.FILLED);
-
-                list.add(poly);
+				// ignore zero-size polygons
+				boolean hasArea;
+				if (ALLOWTINYPOLYGONS) hasArea = true; else
+				{
+					hasArea = false;
+					for(int i=1; i<pointList.size(); i++)
+					{
+						if (pointList.get(i-1).distance(pointList.get(i)) > .00001) { hasArea = true;   break; }
+					}
+				}
+				if (hasArea)
+				{
+					Point2D [] points = new Point2D[pointList.size()];
+	                System.arraycopy(pointList.toArray(), 0, points, 0, pointList.size());
+					PolyBase poly = new PolyBase(points);
+					poly.setLayer(layer);
+					poly.setStyle(Poly.Type.FILLED);
+					list.add(poly);
+				}
 
 				pointList.clear();
 			} else if (type == PathIterator.SEG_MOVETO || type == PathIterator.SEG_LINETO)
@@ -1944,7 +2209,7 @@ public class PolyBase implements Shape, PolyNodeMerge
     }
 
     // This assumes the algorithm starts with external loop
-    public static List<PolyBase> getPointsFromComplex(Area area, Layer layer)
+    private static List<PolyBase> getPointsFromComplex(Area area, Layer layer)
 	{
         List<PolyBase> list = getLoopsFromArea(area, layer);
         List<PolyBaseTree> roots = getTreesFromLoops(list);
