@@ -25,6 +25,7 @@ package com.sun.electric.tool;
 
 import com.sun.electric.database.EditingPreferences;
 import com.sun.electric.database.Environment;
+import com.sun.electric.database.Snapshot;
 import com.sun.electric.database.hierarchy.Cell;
 import com.sun.electric.database.hierarchy.EDatabase;
 import com.sun.electric.database.hierarchy.Library;
@@ -49,6 +50,7 @@ import java.util.Date;
 import java.util.Iterator;
 import java.util.List;
 import java.util.logging.Logger;
+import javax.swing.SwingUtilities;
 
 /**
  * Jobs are processes that will run in the background, such as
@@ -96,7 +98,7 @@ import java.util.logging.Logger;
 public abstract class Job implements Serializable {
 
     private static boolean GLOBALDEBUG = false;
-    private static int recommendedNumThreads;
+    private static boolean threadSafe;
     static final int PROTOCOL_VERSION = 19; // Apr 17
     public static boolean LOCALDEBUGFLAG; // Gilda's case
 //    private static final String CLASS_NAME = Job.class.getName();
@@ -121,6 +123,10 @@ public abstract class Job implements Serializable {
 		/** Describes a server database examination. */	SERVER_EXAMINE,
 
 		/** Describes a client database examination. */ CLIENT_EXAMINE;
+
+        public boolean isExamine() {
+            return this == Job.Type.CLIENT_EXAMINE || this == Job.Type.SERVER_EXAMINE;
+        }
 	}
 
 
@@ -136,7 +142,8 @@ public abstract class Job implements Serializable {
 
 
 	/** default execution time in milis */      /*private*/ public static final int MIN_NUM_SECONDS = 60000;
-    /** job manager */                          /*private*/ static JobManager jobManager;
+    /** job manager */                          /*private*/ static ServerJobManager serverJobManager;
+    /** job manager */                          /*private*/ static ClientJobManager clientJobManager;
 	static AbstractUserInterface currentUI;
     static Thread clientThread;
 
@@ -163,35 +170,40 @@ public abstract class Job implements Serializable {
     transient EJob ejob;
     transient EDatabase database;
 
-    public static void initJobManager(int numThreads, String loggingFilePath, int socketPort, AbstractUserInterface ui, Job initDatabaseJob) {
-        Job.recommendedNumThreads = numThreads;
+    public static void initJobManager(int numThreads, String loggingFilePath, int socketPort, AbstractUserInterface ui, Job initDatabaseJob, boolean threadSafe) {
+        if (threadSafe) {
+            setThreadSafe();
+        }
         currentUI = ui;
-        jobManager = new ServerJobManager(numThreads, loggingFilePath, false, socketPort);
-        jobManager.runLoop(initDatabaseJob);
+        serverJobManager = new ServerJobManager(numThreads, loggingFilePath, false, socketPort);
+        serverJobManager.runLoop(initDatabaseJob);
     }
 
     public static void pipeServer(int numThreads, String loggingFilePath, int socketPort) {
+        setThreadSafe();
         Pref.forbidPreferences();
         EDatabase.setServerDatabase(new EDatabase(IdManager.stdIdManager.getInitialSnapshot()));
         Tool.initAllTools();
-        jobManager = new ServerJobManager(numThreads, loggingFilePath, true, socketPort);
+        serverJobManager = new ServerJobManager(numThreads, loggingFilePath, true, socketPort);
     }
 
     public static void socketClient(String serverMachineName, int socketPort, AbstractUserInterface ui, Job initDatabaseJob) {
+        setThreadSafe();
         currentUI = ui;
         try {
-            jobManager = new ClientJobManager(serverMachineName, socketPort);
-            jobManager.runLoop(initDatabaseJob);
+            clientJobManager = new ClientJobManager(serverMachineName, socketPort);
+            clientJobManager.runLoop(initDatabaseJob);
         } catch (IOException e) {
             e.printStackTrace();
         }
     }
 
     public static void pipeClient(Process process, AbstractUserInterface ui, Job initDatabaseJob, boolean skipOneLine) {
+        setThreadSafe();
         currentUI = ui;
         try {
-            jobManager = new ClientJobManager(process, skipOneLine);
-            jobManager.runLoop(initDatabaseJob);
+            clientJobManager = new ClientJobManager(process, skipOneLine);
+            clientJobManager.runLoop(initDatabaseJob);
         } catch (IOException e) {
             e.printStackTrace();
         }
@@ -214,11 +226,11 @@ public abstract class Job implements Serializable {
 	 */
     public Job(String jobName, Tool tool, Type jobType, Cell upCell, Cell downCell, Priority priority) {
         ejob = new EJob(this, jobType, jobName, EditingPreferences.getThreadEditingPreferences());
-        database = getUserInterface() != null ? getUserInterface().getDatabase() : EDatabase.clientDatabase();
+        UserInterface ui = getUserInterface();
+        database = ui != null ? ui.getDatabase() : EDatabase.clientDatabase();
 		this.tool = tool;
         this.deleteWhenDone = true;
         startTime = endTime = 0;
-        UserInterface ui = getUserInterface();
         Technology curTech = ui != null ? ui.getCurrentTechnology() : null;
         curTechId = curTech != null ? curTech.getId() : null;
         Library curLib = ui != null ? ui.getCurrentLibrary() : null;
@@ -269,23 +281,43 @@ public abstract class Job implements Serializable {
     {
         this.deleteWhenDone = deleteWhenDone;
 
-        Thread currentThread = Thread.currentThread();
-        if (currentThread instanceof EThread && ((EThread)currentThread).ejob.jobType != Job.Type.CLIENT_EXAMINE) {
-            ejob.startedByServer = true;
-            ejob.client = ((EThread)currentThread).ejob.client;
-            ejob.jobKey = ejob.client.newJobId(ejob.startedByServer, true);
+        UserInterface ui = getUserInterface();
+        Job.Key curJobKey = ui.getJobKey();
+        boolean startedByServer = curJobKey.doItOnServer;
+        boolean doItOnServer = ejob.jobType != Job.Type.CLIENT_EXAMINE;
+        if (startedByServer) {
+            assert doItOnServer;
+            ejob.client = Job.serverJobManager.serverConnections.get(curJobKey.clientId);
+            ejob.jobKey = ejob.client.newJobId(startedByServer, doItOnServer);
             ejob.serverJob.startTime = System.currentTimeMillis();
             ejob.serialize(EDatabase.serverDatabase());
             ejob.clientJob = null;
+            Job.serverJobManager.addJob(ejob, onMySnapshot);
         } else {
-            ejob.client = Job.getExtendedUserInterface();
-            ejob.jobKey = ejob.client.newJobId(ejob.startedByServer, false);
+            ejob.client = Job.currentUI;
+            ejob.jobKey = ejob.client.newJobId(startedByServer, doItOnServer);
             ejob.clientJob.startTime = System.currentTimeMillis();
             ejob.serverJob = null;
-            if (ejob.jobType != Job.Type.CLIENT_EXAMINE)
+            if (doItOnServer) {
                 ejob.serialize(EDatabase.clientDatabase());
+                Job.currentUI.putProcessingEJob(ejob, onMySnapshot);
+                if (serverJobManager != null) {
+                    serverJobManager.addJob(ejob, onMySnapshot);
+                } else {
+                    assert SwingUtilities.isEventDispatchThread();
+                    try {
+                        clientJobManager.writeEJob(ejob);
+                    } catch (IOException e) {
+                        e.printStackTrace();
+                    }
+                }
+            } else {
+                Job.currentUI.putProcessingEJob(ejob, onMySnapshot);
+                if (!isThreadSafe()) {
+                    serverJobManager.addJob(ejob, onMySnapshot);
+                }
+            }
         }
-        jobManager.addJob(ejob, onMySnapshot);
     }
 
     /**
@@ -359,7 +391,10 @@ public abstract class Job implements Serializable {
 //    }
 
     protected synchronized void setProgress(String progress) {
-        jobManager.setProgress(ejob, progress);
+        if (serverJobManager != null)
+            serverJobManager.setProgress(ejob, progress);
+        else
+            ejob.progress = progress;
     }
 
     private synchronized String getProgress() { return ejob.progress; }
@@ -419,7 +454,9 @@ public abstract class Job implements Serializable {
 	}
 
     /** get all jobs iterator */
-    public static Iterator<Job> getAllJobs() { return jobManager.getAllJobs(); }
+    public static Iterator<Job> getAllJobs() {
+        return serverJobManager != null ? serverJobManager.getAllJobs() : currentUI.getAllJobs().iterator();
+    }
 
     /**
      * If this current thread is a EThread running a Job return the Job.
@@ -431,16 +468,6 @@ public abstract class Job implements Serializable {
         return thread instanceof EThread ? ((EThread)thread).getRunningJob() : null;
     }
 
-    /**
-     * If this current thread is a EThread running a Job return the EJob.
-     * Return null otherwise.
-     * @return a running Job or null
-     */
-    static EJob getRunningEJob() {
-        Thread thread = Thread.currentThread();
-        return thread instanceof EThread ? ((EThread)thread).getRunningEJob() : null;
-    }
-
     /**
      * Returns true if this current thread is a EThread running a server Job.
      * @return true if this current thread is a EThread running a server Job.
@@ -483,15 +510,36 @@ public abstract class Job implements Serializable {
             //System.out.println("Cannot delete running jobs.  Wait till finished or abort");
             return false;
         }
-        jobManager.removeJob(this);
+        if (serverJobManager != null)
+            serverJobManager.removeJob(this);
+        else
+           currentUI.removeProcessingEJob(getKey());
         return true;
     }
 
+    private static final ThreadLocal<UserInterface> threadUserInterface = new ThreadLocal<UserInterface>() {
+//        @Override
+//        protected UserInterface initialValue() {
+//            throw new IllegalStateException();
+//        }
+    };
+
     public static UserInterface getUserInterface() {
         Thread currentThread = Thread.currentThread();
         if (currentThread instanceof EThread)
             return ((EThread)currentThread).getUserInterface();
-        return currentUI;
+        else if (currentThread == clientThread)
+            return currentUI;
+        else
+            return threadUserInterface.get();
+    }
+
+    public static void setUserInterface(UserInterface ui) {
+        Thread currentThread = Thread.currentThread();
+        assert !(currentThread instanceof EThread) && currentThread != clientThread;
+        if (ui == null)
+            throw new UnsupportedOperationException();
+        threadUserInterface.set(ui);
     }
 
     /**
@@ -537,6 +585,22 @@ public abstract class Job implements Serializable {
         currentUI.updateIncrementalDRCErrors(cell, newErrors, delErrors);
     }
 
+    /**
+     * Find some valid snapshot in cache.
+     * @return some valid snapshot
+     */
+    public static Snapshot findValidSnapshot() {
+        return EThread.findValidSnapshot();
+    }
+
+    public static boolean isThreadSafe() {
+        return threadSafe;
+    }
+
+    static void setThreadSafe() {
+        threadSafe = true;
+    }
+
 	//-------------------------------JOB UI--------------------------------
 
     public String toString() { return ejob.jobName+" ("+getStatus()+")"; }
@@ -621,6 +685,10 @@ public abstract class Job implements Serializable {
             this(client.connectionId, jobId, doItOnServer);
         }
 
+        public boolean startedByServer() {
+            return jobId > 0;
+        }
+
         @Override
         public boolean equals(Object o) {
             if (o == this) return true;
