@@ -39,6 +39,7 @@ public class SCLibraryGen {
     private Variable.Key sizeKey = Variable.findKey("ATTR_X");
 
     public static final Variable.Key STANDARDCELL = Variable.newKey("ATTR_StandardCell");
+    public static final Variable.Key DONOTEXTRACTFORDSPF = Variable.newKey("ATTR_ExcludeFromDSPFExtraction");
 
     private static final int blueColorIndex = EGraphics.makeIndex(Color.blue);
 
@@ -157,7 +158,9 @@ public class SCLibraryGen {
 
                 schcell = scLibrary.findNodeProto(cellname+"{sch}");
                 // mark schematic as standard cell
-                markStandardCell(schcell);
+                List<Cell> cells = new ArrayList<Cell>();
+                cells.add(schcell);
+                markStandardCell(cells, null);
             }
         }
         return true;
@@ -255,40 +258,59 @@ public class SCLibraryGen {
     /**
      * Mark the cell as a standard cell.
      * This version of the method performs the task in a Job.
-     * @param cell the cell to mark with the standard cell attribute marker
+     * @param standardCells a list of Cells to mark with the standard cell attribute marker.
+     * @param notStandardCells a list of Cells to remove the standard cell attribute marker.
      */
-    public static void markStandardCellJob(Cell cell) {
-        CreateVar job = new CreateVar(cell, STANDARDCELL, new Integer(1));
+    public static void markStandardCellJob(List<Cell> standardCells, List<Cell> notStandardCells) {
+        CreateVar job = new CreateVar(standardCells, notStandardCells, STANDARDCELL);
         job.startJob();
     }
 
     /**
      * Mark the cell as a standard cell
-     * @param cell the cell to mark with the standard cell attribute marker
+     * @param standardCells a list of Cells to mark with the standard cell attribute marker.
+     * @param notStandardCells a list of Cells to remove the standard cell attribute marker.
      */
-    public static void markStandardCell(Cell cell) {
-        CreateVar job = new CreateVar(cell, STANDARDCELL, new Integer(1));
+    public static void markStandardCell(List<Cell> standardCells, List<Cell> notStandardCells) {
+        CreateVar job = new CreateVar(standardCells, notStandardCells, STANDARDCELL);
         job.doIt();
     }
 
-    public static class CreateVar extends Job {
-        private Cell cell;
-        private Variable.Key key;
-        private Object defaultVal;
-        public CreateVar(Cell cell, Variable.Key key, Object defaultVal) {
+    private static class CreateVar extends Job
+    {
+        private List<Cell> standardCells;
+        private List<Cell> notStandardCells;
+        private Variable.Key varName;
+
+        public CreateVar(List<Cell> standardCells, List<Cell> notStandardCells, Variable.Key varName)
+        {
             super("Create Var", User.getUserTool(), Job.Type.CHANGE, null, null, Job.Priority.USER);
-            this.cell = cell;
-            this.key = key;
-            this.defaultVal = defaultVal;
+            this.standardCells = standardCells;
+            this.notStandardCells = notStandardCells;
+            this.varName = varName;
         }
-        public boolean doIt() {
+
+        public boolean doIt()
+        {
             TextDescriptor td = TextDescriptor.getCellTextDescriptor().withInterior(true).withDispPart(TextDescriptor.DispPos.NAMEVALUE);
-            if (cell == null) return false;
-            cell.newVar(key, defaultVal, td);
+            if (standardCells != null)
+            	for(Cell cell : standardCells)
+            		cell.newVar(varName, new Integer(1), td);
+            if (notStandardCells != null)
+	            for(Cell cell : notStandardCells)
+	            	cell.delVar(varName);
             return true;
         }
     }
 
+    /**
+     * Mark the cell to not be extracted for DSPF
+     * @param cells the cells
+     */
+    public static void markDoNotExtractForDSPFJob(List<Cell> cells) {
+        CreateVar job = new CreateVar(cells, null, DONOTEXTRACTFORDSPF);
+        job.startJob();
+    }
 
     /**
      * Return the standard cells in a hierarchy starting from
@@ -314,6 +336,18 @@ public class SCLibraryGen {
         return cell.getVar(STANDARDCELL) != null;
     }
 
+    /**
+     * Returns true if the cell is marked to not be extracted for DSPF,
+     * for Static Timing Analysis
+     * @param cell the cell to check
+     * @return true if marked to not extract for DSPF
+     */
+    public static boolean isMarkedDoNotExtractForDSPF(Cell cell) {
+        Cell schcell = cell.getCellGroup().getMainSchematics();
+        if (schcell != null) cell = schcell;
+        return cell.getVar(DONOTEXTRACTFORDSPF) != null;
+    }
+
     private void prErr(String msg) {
         System.out.println("Standard Cell Library Generator Error: "+msg);
     }
@@ -335,15 +369,23 @@ public class SCLibraryGen {
         private Map<Cell,Integer> standardCellMap = new HashMap<Cell,Integer>();
         private Map<String,Cell> standardCellsByName = new HashMap<String,Cell>();
         private List<VarContext> standardCellContexts = new ArrayList<VarContext>();
+        private Map<VarContext,VarContext> emptyCellContexts = new HashMap<VarContext,VarContext>();
+        private Map<VarContext,VarContext> containsStandardCellContexts = new HashMap<VarContext,VarContext>();
+        private Set<Cell> doNotExtractForDSPF = new HashSet<Cell>();
         private boolean nameConflict = false;
 
         public boolean enterCell(HierarchyEnumerator.CellInfo info) {
             Cell cell = info.getCell();
+
+            if (isMarkedDoNotExtractForDSPF(cell)) doNotExtractForDSPF.add(cell);
+
             // skip cached and does not contain standard cell
             // (we want to traverse all hierarchy that contains standard cells
             // to produce the standard cell contexts list)
-            if (standardCellMap.get(cell) == doesNotContainStandardCell)
+            if (standardCellMap.get(cell) == doesNotContainStandardCell) {
+                emptyCellContexts.put(info.getContext(), info.getContext());
                 return false;
+            }
 
             if (SCLibraryGen.isStandardCell(cell)) {
                 standardCellContexts.add(info.getContext());
@@ -367,6 +409,7 @@ public class SCLibraryGen {
 
         public void exitCell(HierarchyEnumerator.CellInfo info) {
             Cell cell = info.getCell();
+            VarContext context = info.getContext();
 
             for (Iterator<NodeInst> it = cell.getNodes(); it.hasNext(); ) {
                 NodeInst ni = it.next();
@@ -377,10 +420,23 @@ public class SCLibraryGen {
                 if (proto == null) proto = (Cell)ni.getProto();
                 if (containsStandardCell(proto) || standardCellMap.get(proto) == standardCell) {
                     standardCellMap.put(cell, containsStandardCell);
+                    containsStandardCellContexts.put(context, context);
                     return;
                 }
             }
             standardCellMap.put(cell, doesNotContainStandardCell);
+            emptyCellContexts.put(context, context);
+            for (Iterator<NodeInst> it = cell.getNodes(); it.hasNext(); ) {
+                NodeInst ni = it.next();
+                if (!ni.isCellInstance()) continue;
+                if (ni.isIconOfParent()) continue;
+                Cell proto = ni.getProtoEquivalent();
+                if (proto == null) proto = (Cell)ni.getProto();
+                if (standardCellMap.get(proto) == doesNotContainStandardCell) {
+                    VarContext nicontext = context.push(ni);
+                    emptyCellContexts.remove(nicontext);
+                }
+            }
         }
 
         public boolean visitNodeInst(Nodable ni, HierarchyEnumerator.CellInfo info) {
@@ -439,6 +495,39 @@ public class SCLibraryGen {
             return standardCellContexts;
         }
 
+        public Set<String> getContainsStandardCellContextsInHier() {
+            Set<VarContext> contexts = containsStandardCellContexts.keySet();
+            Set<String> sorted = new TreeSet<String>();
+            for (VarContext context : contexts) {
+                String s = context.getInstPath("/");
+                sorted.add(s);
+            }
+            return sorted;
+        }
+
+        /**
+         * Get the contexts of cells that do not contain standard cells
+         * in the hierarchy. This set is minimal.
+         * @return a set of var contexts
+         */
+        public Set<String> getEmptyCellContextsInHier() {
+            Set<VarContext> contexts = emptyCellContexts.keySet();
+            Set<String> sorted = new TreeSet<String>();
+            for (VarContext context : contexts) {
+                String s = context.getInstPath("/");
+                sorted.add(s);
+            }
+            return sorted;
+        }
+
+        /**
+         * Get cells that have been marked to not be extracted for DSPF
+         * @return the cells
+         */
+        public Set<Cell> getCellsMarkedToNotExtractForDSPF() {
+            return doNotExtractForDSPF;
+        }
+
         /**
          * Get cells of the given type. The type is one of
          * Type.StandardCell
