@@ -51,6 +51,9 @@ import com.sun.electric.tool.Job;
 import com.sun.electric.tool.JobException;
 import com.sun.electric.tool.user.ui.ClickZoomWireListener;
 import com.sun.electric.tool.user.ui.EditWindow;
+import com.sun.electric.tool.user.ui.MessagesWindow;
+import com.sun.electric.tool.user.ui.TextWindow;
+import com.sun.electric.tool.user.ui.TopLevel;
 import com.sun.electric.tool.user.ui.WindowFrame;
 
 import java.awt.event.ActionEvent;
@@ -85,7 +88,10 @@ import javax.swing.KeyStroke;
 public class Clipboard //implements ClipboardOwner
 {
     /** The Clipboard CellId. */
-    public static final CellId clipCellId = IdManager.stdIdManager.newLibId("Clipboard!!").newCellId(CellName.parseName("Clipboard!!;1{}"));
+    public static final String CLIPBOARD_LIBRAY_NAME = "Clipboard!!";
+    public static final String CLIPBOARD_CELL_NAME =  "Clipboard!!;1{}";
+    public static final int CLIPBOARD_CELL_INDEX = 0;
+
 //	/** the last node that was duplicated */			private static NodeInst  lastDup = null;
 	/** the amount that the last node moved */			private static double    lastDupX = 10, lastDupY = 10;
 //    private static final Clipboard clip = new Clipboard();
@@ -118,7 +124,24 @@ public class Clipboard //implements ClipboardOwner
         // Clear text buffer
         TextUtils.setTextOnClipboard(null);
 
-        // see what is highlighted
+        // is this the messages window?
+        MessagesWindow mw = TopLevel.getMessagesWindow();
+        if (mw.isFocusOwner())
+        {
+        	mw.copyText(false, false);
+        	return;
+        }
+
+        // if current window is text window, copy from it
+		WindowFrame wf = WindowFrame.getCurrentWindowFrame();
+		if (wf != null && wf.getContent() instanceof TextWindow)
+		{
+			TextWindow tw = (TextWindow)wf.getContent();
+			tw.copy();
+			return;
+		}
+
+		// get the edit window
 		EditWindow wnd = EditWindow.needCurrent();
 		if (wnd == null) return;
 		Highlighter highlighter = wnd.getHighlighter();
@@ -164,7 +187,24 @@ public class Clipboard //implements ClipboardOwner
 	 */
 	public static void cut()
 	{
-		// see what is highlighted
+        // is this the messages window?
+        MessagesWindow mw = TopLevel.getMessagesWindow();
+        if (mw.isFocusOwner())
+        {
+        	mw.copyText(false, true);
+        	return;
+        }
+
+        // if current window is text window, cut from it
+		WindowFrame wf = WindowFrame.getCurrentWindowFrame();
+		if (wf != null && wf.getContent() instanceof TextWindow)
+		{
+			TextWindow tw = (TextWindow)wf.getContent();
+			tw.cut();
+			return;
+		}
+
+		// get the edit window
 		EditWindow wnd = EditWindow.needCurrent();
 		if (wnd == null) return;
 		Highlighter highlighter = wnd.getHighlighter();
@@ -212,6 +252,23 @@ public class Clipboard //implements ClipboardOwner
 	 */
 	public static void paste()
 	{
+        // is this the messages window?
+        MessagesWindow mw = TopLevel.getMessagesWindow();
+        if (mw.isFocusOwner())
+        {
+        	mw.pasteText();
+        	return;
+        }
+
+        // if current window is text window, paste to it
+		WindowFrame wf = WindowFrame.getCurrentWindowFrame();
+		if (wf != null && wf.getContent() instanceof TextWindow)
+		{
+			TextWindow tw = (TextWindow)wf.getContent();
+			tw.paste();
+			return;
+		}
+
         // Get text put in the clipboard but using variables.
         String extraText = TextUtils.getTextOnClipboard();
 
@@ -331,11 +388,15 @@ public class Clipboard //implements ClipboardOwner
 			EventListener currentListener = WindowFrame.getListener();
 			WindowFrame.setListener(new PasteListener(wnd, geomList, textList, currentListener,
 				inPlace, inPlaceOrient, false));
-		} else
+		} else if (User.isDuplicateInPlace()) {
+			new PasteObjects(parent, geomList, textList, 0, 0,
+				User.getAlignmentToGrid(), User.isDupCopiesExports(), User.isIncrementRightmostIndex(),
+				User.isArcsAutoIncremented(), inPlace, inPlaceOrient);
+        } else
 		{
 			new PasteObjects(parent, geomList, textList, lastDupX, lastDupY,
-				User.getAlignmentToGrid(), User.isDupCopiesExports(), User.isArcsAutoIncremented(),
-				inPlace, inPlaceOrient);
+				User.getAlignmentToGrid(), User.isDupCopiesExports(), User.isIncrementRightmostIndex(),
+				User.isArcsAutoIncremented(), inPlace, inPlaceOrient);
 		}
 	}
 
@@ -368,6 +429,10 @@ public class Clipboard //implements ClipboardOwner
 		{
 			EventListener currentListener = WindowFrame.getListener();
 			WindowFrame.setListener(new PasteListener(wnd, geomList, textList, currentListener, null, null, true));
+		} else if (User.isDuplicateInPlace()) {
+            lastDupX = 0;
+            lastDupY = 0;
+			new DuplicateObjects(wnd.getCell(), geomList, textList, User.getAlignmentToGrid());
 		} else
 		{
 			new DuplicateObjects(wnd.getCell(), geomList, textList, User.getAlignmentToGrid());
@@ -584,6 +649,7 @@ public class Clipboard //implements ClipboardOwner
 		private Cell cell;
 		private List<Geometric> geomList, newGeomList;
 		private List<DisplayedText> textList, newTextList;
+		private boolean dupCopiesExports, fromRight, autoIncrementArcs;
 		private Dimension2D alignment;
 
 		protected DuplicateObjects(Cell cell, List<Geometric> geomList, List<DisplayedText> textList, Dimension2D alignment)
@@ -593,6 +659,9 @@ public class Clipboard //implements ClipboardOwner
 			this.geomList = geomList;
 			this.textList = textList;
 			this.alignment = alignment;
+			dupCopiesExports = User.isDupCopiesExports();
+			fromRight = User.isIncrementRightmostIndex();
+			autoIncrementArcs = User.isArcsAutoIncremented();
 			startJob();
 		}
 
@@ -601,9 +670,8 @@ public class Clipboard //implements ClipboardOwner
 			// copy objects to clipboard
 			newGeomList = new ArrayList<Geometric>();
 			newTextList = new ArrayList<DisplayedText>();
-			copyListToCell(cell, geomList, textList, newGeomList, newTextList,
-				new Point2D.Double(lastDupX, lastDupY), User.isDupCopiesExports(), User.isArcsAutoIncremented(),
-				alignment, null, null);
+			copyListToCell(cell, geomList, textList, newGeomList, newTextList, new Point2D.Double(lastDupX, lastDupY),
+				dupCopiesExports, fromRight, autoIncrementArcs, alignment, null, null);
 			fieldVariableChanged("newGeomList");
 			fieldVariableChanged("newTextList");
 			return true;
@@ -707,12 +775,12 @@ public class Clipboard //implements ClipboardOwner
 		private List<DisplayedText> textList, newTextList;
 		private double dX, dY;
 		private Dimension2D alignment;
-		private boolean copyExports, uniqueArcs;
+		private boolean copyExports, fromRight, uniqueArcs;
 		private AffineTransform inPlace;
 		private Orientation inPlaceOrient;
 
 		protected PasteObjects(Cell cell, List<Geometric> geomList, List<DisplayedText> textList,
-			double dX, double dY, Dimension2D alignment, boolean copyExports, boolean uniqueArcs,
+			double dX, double dY, Dimension2D alignment, boolean copyExports, boolean fromRight, boolean uniqueArcs,
 			AffineTransform inPlace, Orientation inPlaceOrient)
 		{
 			super("Paste", User.getUserTool(), Job.Type.CHANGE, null, null, Job.Priority.USER);
@@ -723,6 +791,7 @@ public class Clipboard //implements ClipboardOwner
 			this.dY = dY;
 			this.alignment = alignment;
 			this.copyExports = copyExports;
+			this.fromRight = fromRight;
 			this.uniqueArcs = uniqueArcs;
 			this.inPlace = inPlace;
 			this.inPlaceOrient = inPlaceOrient;
@@ -738,7 +807,7 @@ public class Clipboard //implements ClipboardOwner
 			newGeomList = new ArrayList<Geometric>();
 			newTextList = new ArrayList<DisplayedText>();
 			copyListToCell(cell, geomList, textList, newGeomList, newTextList,
-				new Point2D.Double(dX, dY), copyExports, uniqueArcs, alignment, inPlace, inPlaceOrient);
+				new Point2D.Double(dX, dY), copyExports, fromRight, uniqueArcs, alignment, inPlace, inPlaceOrient);
 			fieldVariableChanged("newGeomList");
 			fieldVariableChanged("newTextList");
 			return true;
@@ -796,12 +865,16 @@ public class Clipboard //implements ClipboardOwner
 
     /****************************** CHANGE JOB SUPPORT ******************************/
 
+    public static CellId getClipCellId() {
+        return IdManager.stdIdManager.getCellId(CLIPBOARD_CELL_INDEX);
+    }
+
 	/**
 	 * Method to clear the clipboard.
 	 */
 	private static Cell getClipCell()
 	{
-        return Job.getUserInterface().getDatabase().getCell(clipCellId);
+        return Job.getUserInterface().getDatabase().getCell(getClipCellId());
 	}
 
 	/**
@@ -843,10 +916,18 @@ public class Clipboard //implements ClipboardOwner
 		}
 	}
 
+	/**
+	 * TODO: DOES THIS VIOLATE PREFERENCE ACCESS?
+	 * @param geomList
+	 * @param textList
+	 * @param alignment
+	 * @param inPlace
+	 * @param inPlaceOrient
+	 */
 	public static void copyListToClipboard(List<Geometric> geomList, List<DisplayedText> textList,
 		Dimension2D alignment, AffineTransform inPlace, Orientation inPlaceOrient) {
 		copyListToCell(getClipCell(), geomList, textList, null, null, new Point2D.Double(),
-				User.isDupCopiesExports(), User.isArcsAutoIncremented(),
+				User.isDupCopiesExports(), User.isIncrementRightmostIndex(), User.isArcsAutoIncremented(),
 				alignment, inPlace, inPlaceOrient);
 	}
 
@@ -859,6 +940,7 @@ public class Clipboard //implements ClipboardOwner
 	 * @param newTextList the list of text objects that were created.
 	 * @param delta an offset for all of the copied Geometrics.
 	 * @param copyExports true to copy exports.
+	 * @param fromRight true to increment the rightmost index of multidimensional arrays.
 	 * @param uniqueArcs true to generate unique arc names.
 	 * @param alignment the grid alignment to use (0 for none).
 	 * @param inPlace the transformation to use which accounts for "down in place" editing.
@@ -867,12 +949,34 @@ public class Clipboard //implements ClipboardOwner
 	 */
 	public static NodeInst copyListToCell(Cell toCell, List<Geometric> geomList, List<DisplayedText> textList,
 		List<Geometric> newGeomList, List<DisplayedText> newTextList, Point2D delta, boolean copyExports,
-		boolean uniqueArcs, Dimension2D alignment, AffineTransform inPlace, Orientation inPlaceOrient)
+		boolean fromRight, boolean uniqueArcs, Dimension2D alignment, AffineTransform inPlace, Orientation inPlaceOrient)
 	{
 		// make a list of all objects to be copied (includes end points of arcs)
 		List<NodeInst> theNodes = new ArrayList<NodeInst>();
 		List<ArcInst> theArcs = new ArrayList<ArcInst>();
-		for (Geometric geom : geomList)
+
+        if (copyExports)
+        {
+        	// copy the geometry list since it might be augmented
+        	List<Geometric> internalGeomList = new ArrayList<Geometric>();
+        	for(Geometric geom : geomList) internalGeomList.add(geom);
+        	geomList = internalGeomList;
+
+        	for(DisplayedText dt : textList)
+            {
+                ElectricObject eObj = dt.getElectricObject();
+                if (eObj instanceof Export)
+                {
+                    // add the object exported to the list
+                    Export e = (Export)eObj;
+                    NodeInst ni = e.getOriginalPort().getNodeInst();
+                    if (!geomList.contains(ni))
+                        geomList.add(ni);
+                }
+            }
+        }
+
+        for (Geometric geom : geomList)
 		{
 			if (geom instanceof NodeInst)
 			{
@@ -921,7 +1025,7 @@ public class Clipboard //implements ClipboardOwner
 			double height = ni.getYSize();
 			String name = null;
 			if (ni.isUsernamed())
-				name = ElectricObject.uniqueObjectName(ni.getName(), toCell, NodeInst.class, false);
+				name = ElectricObject.uniqueObjectName(ni.getName(), toCell, NodeInst.class, false, fromRight);
 			EPoint point = new EPoint(ni.getAnchorCenterX()+dX, ni.getAnchorCenterY()+dY);
 			Orientation orient = ni.getOrient();
 			if (inPlace != null)
@@ -938,7 +1042,7 @@ public class Clipboard //implements ClipboardOwner
 				System.out.println("Cannot create node");
 				return lastCreatedNode;
 			}
-			newNi.copyStateBits(ni);
+			newNi.copyStateBitsAndExpandedFlag(ni);
 			newNi.copyTextDescriptorFrom(ni, NodeInst.NODE_PROTO);
 			newNi.copyTextDescriptorFrom(ni, NodeInst.NODE_NAME);
 			newNi.copyVarsFrom(ni);
@@ -967,7 +1071,7 @@ public class Clipboard //implements ClipboardOwner
 				reExpThese.add(newPi);
 				originalExports.put(newPi, e);
 			}
-			ExportChanges.reExportPorts(toCell, reExpThese, false, true, true, false, originalExports);
+			ExportChanges.reExportPorts(toCell, reExpThese, false, true, true, false, fromRight, originalExports);
 		}
 
 		Map<ArcInst,ArcInst> newArcs = new HashMap<ArcInst,ArcInst>();
@@ -1020,7 +1124,7 @@ public class Clipboard //implements ClipboardOwner
 						String newName = newArcNames.get(name);
 						if (newName == null)
 						{
-							newName = ElectricObject.uniqueObjectName(name, toCell, ArcInst.class, false);
+							newName = ElectricObject.uniqueObjectName(name, toCell, ArcInst.class, false, fromRight);
 							newArcNames.put(name, newName);
 						}
 						name = newName;
@@ -1105,8 +1209,7 @@ public class Clipboard //implements ClipboardOwner
 		destNode = CircuitChangeJobs.replaceNodeInst(destNode, srcNode.getProto(), true, false);
 		if (destNode == null) return null;
 
-		destNode.clearExpanded();
-		if (srcNode.isExpanded()) destNode.setExpanded();
+		destNode.setExpanded(srcNode.isExpanded());
 
 		if (!destNode.isCellInstance() && !srcNode.isCellInstance()) {
 			if (srcNode.getProto().getTechnology() == destNode.getProto().getTechnology()) {
@@ -1156,8 +1259,8 @@ public class Clipboard //implements ClipboardOwner
 		destNode.copyVarsFrom(srcNode);
 
 		// copy any special user bits
-		destNode.copyStateBits(srcNode);
-		destNode.clearExpanded();
+		destNode.copyStateBitsAndExpandedFlag(srcNode);
+		destNode.setExpanded(false);
 		destNode.clearLocked();
 
 		return(destNode);
@@ -1534,8 +1637,8 @@ public class Clipboard //implements ClipboardOwner
 			Cell cell = WindowFrame.needCurCell();
 			if (cell != null)
 				new PasteObjects(cell, geomList, textList, delta.getX(), delta.getY(),
-					User.getAlignmentToGrid(), User.isDupCopiesExports(), User.isArcsAutoIncremented(),
-					inPlace, inPlaceOrient);
+					User.getAlignmentToGrid(), User.isDupCopiesExports(), User.isIncrementRightmostIndex(),
+					User.isArcsAutoIncremented(), inPlace, inPlaceOrient);
 		}
 
 		public void mouseMoved(MouseEvent evt)
