@@ -24,6 +24,8 @@
 package com.sun.electric.tool.user;
 
 import com.sun.electric.database.Environment;
+import com.sun.electric.tool.Job;
+import com.sun.electric.tool.UserInterfaceExec;
 import java.io.*;
 import java.util.ArrayList;
 import java.util.List;
@@ -53,6 +55,7 @@ public class Exec extends Thread {
         private InputStream in;
         private OutputStream redirect;
         private char [] buf;
+        private final UserInterfaceExec userInterface = new UserInterfaceExec();
 
         /**
          * Create a stream reader that will read from the stream
@@ -76,7 +79,11 @@ public class Exec extends Thread {
         }
 
         public void run() {
-            try {
+                if (Thread.currentThread() == this) {
+//                    Environment.setThreadEnvironment(launcherEnvironment);
+                    Job.setUserInterface(userInterface);
+                }
+                    try {
                 PrintWriter pw = null;
                 if (redirect != null) pw = new PrintWriter(redirect);
 
@@ -136,6 +143,7 @@ public class Exec extends Thread {
     private final String [] envVars;
     private final File dir;                       // working directory
     private final Environment launcherEnvironment;
+    private final UserInterfaceExec userInterface;
     private final OutputStream outStreamRedir;    // output of process redirected to this stream
     private final OutputStream errStreamRedir;    // error messages of process redirected to this stream
     private PrintWriter processWriter;      // connect to input of process
@@ -164,6 +172,7 @@ public class Exec extends Thread {
         this.envVars = envVars;
         this.dir = dir;
         launcherEnvironment = Environment.getThreadEnvironment();
+        userInterface = new UserInterfaceExec();
         this.outStreamRedir = outStreamRedir;
         this.errStreamRedir = errStreamRedir;
         this.processWriter = null;
@@ -187,6 +196,7 @@ public class Exec extends Thread {
         this.envVars = envVars;
         this.dir = dir;
         launcherEnvironment = Environment.getThreadEnvironment();
+        userInterface = new UserInterfaceExec();
         this.outStreamRedir = outStreamRedir;
         this.errStreamRedir = errStreamRedir;
         this.processWriter = null;
@@ -196,7 +206,10 @@ public class Exec extends Thread {
     }
 
     public void run() {
-        Environment.setThreadEnvironment(launcherEnvironment);
+        if (Thread.currentThread() == this) {
+            Environment.setThreadEnvironment(launcherEnvironment);
+            Job.setUserInterface(userInterface);
+        }
         if (outStreamRedir instanceof OutputStreamChecker) {
             ((OutputStreamChecker)outStreamRedir).setExec(this);
         }
