@@ -23,14 +23,23 @@
  */
 package com.sun.electric.tool.user;
 
-import com.sun.electric.tool.user.menus.EMenuBar;
+import com.sun.electric.tool.user.dialogs.EDialog;
+import com.sun.electric.tool.user.dialogs.EModelessDialog;
+import com.sun.electric.tool.user.dialogs.GetInfoText;
+import com.sun.electric.tool.user.dialogs.OpenFile;
+import com.sun.electric.tool.user.help.ManualViewer;
+import com.sun.electric.tool.user.ui.EditWindow;
 import com.sun.electric.tool.user.ui.KeyBindings;
 import com.sun.electric.tool.user.ui.KeyStrokePair;
+import com.sun.electric.tool.user.ui.TextWindow;
 import com.sun.electric.tool.user.ui.TopLevel;
+import com.sun.electric.tool.user.ui.WindowContent;
 import com.sun.electric.tool.user.ui.WindowFrame;
+import com.sun.electric.tool.user.waveform.WaveformWindow;
 
-import java.awt.Menu;
-import java.awt.MenuBar;
+import java.awt.Component;
+import java.awt.KeyEventDispatcher;
+import java.awt.KeyboardFocusManager;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.InputEvent;
@@ -49,9 +58,10 @@ import java.util.prefs.Preferences;
 import javax.swing.AbstractAction;
 import javax.swing.ActionMap;
 import javax.swing.InputMap;
+import javax.swing.JDialog;
 import javax.swing.JMenu;
 import javax.swing.JMenuBar;
-import javax.swing.JPopupMenu;
+import javax.swing.JOptionPane;
 import javax.swing.KeyStroke;
 
 /**
@@ -83,7 +93,8 @@ import javax.swing.KeyStroke;
  *
  * @author  gainsley
  */
-public class KeyBindingManager {
+public class KeyBindingManager implements KeyEventDispatcher
+{
 
     // ----------------------------- object stuff ---------------------------------
     /** Hash table of lists all key bindings */     private Map<KeyStroke,Set<String>> inputMap;
@@ -119,7 +130,7 @@ public class KeyBindingManager {
 
         // register this with KeyboardFocusManager
         // so we receive all KeyEvents
-        //KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(this);
+        KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(this);
         //KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventPostProcessor(this);
 
         // add to list of all managers
@@ -129,6 +140,11 @@ public class KeyBindingManager {
         initialize();
     }
 
+    public boolean dispatchKeyEvent(KeyEvent e)
+    {
+    	return processKeyEvent(e);
+    }
+
     /**
      * Called when disposing of this manager, allows memory used to
      * be reclaimed by removing static references to this.
@@ -395,26 +411,27 @@ public class KeyBindingManager {
         // see if this is a valid key event
         if (!validKeyEvent(e)) return false;
 
-        // get KeyStroke
-		KeyStroke stroke = KeyStroke.getKeyStrokeForEvent(e);
-		if (DEBUG) System.out.println("  Current key is "+stroke+", code="+e.getKeyCode()+", type="+stroke.getKeyEventType());
-
-		// convert arrow keys which appear only on release event
-		if ((e.getModifiers() & (InputEvent.CTRL_MASK|InputEvent.SHIFT_MASK)) != InputEvent.CTRL_MASK &&
-			stroke.getKeyEventType() == KeyEvent.KEY_RELEASED)
-		{
-			if (e.getKeyCode() == KeyEvent.VK_LEFT || e.getKeyCode() == KeyEvent.VK_RIGHT ||
-				e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN)
-			{
-				stroke = KeyStroke.getKeyStroke(e.getKeyCode(), stroke.getModifiers());
-			}
-		} else
-		{
-			// remove shift modifier from Events.  Lets KeyStrokes like '<' register correctly,
-			// because they are always delivered as SHIFT-'<'.
-			if ((e.getModifiers() & InputEvent.SHIFT_MASK) != 0 && !Character.isLetter(e.getKeyCode()) && !Character.isDigit(e.getKeyCode()))
-				stroke = KeyStroke.getKeyStroke(e.getKeyChar());
-		}
+        // ignore events that come from dialogs, or non-Control events that are not from an EditWindow/WaveformWindow
+        Component c = e.getComponent();
+        boolean valid = false;
+        while (c != null)
+        {
+            if (c instanceof EditWindow) { valid = true;   break; }
+            if (c instanceof WaveformWindow.OnePanel) { valid = true;   break; }
+            if (c instanceof TopLevel) { valid = true;   break; }
+        	if (c instanceof EDialog) { lastPrefix = null;  return false; }
+        	if (c instanceof EModelessDialog) { lastPrefix = null;  return false; }
+        	if (c instanceof JOptionPane) { lastPrefix = null;  return false; }
+        	if (c instanceof TextWindow.TextWindowPanel) { lastPrefix = null;  return false; }
+        	if (c instanceof OpenFile.OpenFileSwing) { lastPrefix = null;  return false; }
+            if (c instanceof GetInfoText.EIPEditorPane) { lastPrefix = null;  return false; }
+            if (c instanceof GetInfoText.EIPTextField) { lastPrefix = null;  return false; }
+            if (c instanceof ManualViewer) { lastPrefix = null;  return false; }
+            if (c instanceof ManualViewer.EditHTML) { lastPrefix = null;  return false; }
+        	c = c.getParent();
+        }
+        if (!valid && (e.getModifiers() & InputEvent.CTRL_MASK) == 0)
+        	{ lastPrefix = null;   return false; }
 
 		// see if any popup menus are visible
 		WindowFrame wf = WindowFrame.getCurrentWindowFrame();
@@ -428,6 +445,33 @@ public class KeyBindingManager {
             return false;
         }
 
+        // get KeyStroke
+		KeyStroke stroke = KeyStroke.getKeyStrokeForEvent(e);
+		if (DEBUG) System.out.println("  Current key is "+stroke+", code="+e.getKeyCode()+", type="+stroke.getKeyEventType());
+
+		// convert arrow keys which appear only on release event
+//		if ((e.getModifiers() & (InputEvent.CTRL_MASK|InputEvent.SHIFT_MASK)) != InputEvent.CTRL_MASK &&
+//			stroke.getKeyEventType() == KeyEvent.KEY_RELEASED)
+//		{
+//			if (e.getKeyCode() == KeyEvent.VK_LEFT || e.getKeyCode() == KeyEvent.VK_RIGHT ||
+//				e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN)
+//			{
+//				stroke = KeyStroke.getKeyStroke(e.getKeyCode(), stroke.getModifiers());
+//			}
+//		} else
+		{
+			// remove shift modifier from Events.  Lets KeyStrokes like '<' register correctly,
+			// because they are always delivered as SHIFT-'<'.
+			if ((e.getModifiers() & InputEvent.SHIFT_MASK) != 0 && !Character.isLetter(e.getKeyCode()) && !Character.isDigit(e.getKeyCode()))
+			{
+				if (e.getKeyCode() != KeyEvent.VK_LEFT && e.getKeyCode() != KeyEvent.VK_RIGHT &&
+					e.getKeyCode() != KeyEvent.VK_UP && e.getKeyCode() != KeyEvent.VK_DOWN)
+						stroke = KeyStroke.getKeyStroke(e.getKeyChar());
+			}
+			if (e.getKeyCode() == '[' || e.getKeyCode() == ']')
+				stroke = KeyStroke.getKeyStroke(e.getKeyChar());
+		}
+
         // ignore if consumed
         if (e.isConsumed()) {
             lastPrefix = null;              // someone did something with it, null prefix key
