@@ -34,6 +34,7 @@ import com.sun.electric.database.variable.ElectricObject;
 import com.sun.electric.database.variable.MutableTextDescriptor;
 import com.sun.electric.database.variable.TextDescriptor;
 import com.sun.electric.database.variable.Variable;
+import com.sun.electric.technology.Technology;
 import com.sun.electric.technology.technologies.Generic;
 import com.sun.electric.tool.Job;
 import com.sun.electric.tool.JobException;
@@ -84,6 +85,14 @@ public class TextInfoPanel extends javax.swing.JPanel
     	this.updateChangesInstantly = updateChangesInstantly;
         initComponents();
 
+		// make all text fields select-all when entered
+	    EDialog.makeTextFieldSelectAllOnTab(pointsSize);
+	    EDialog.makeTextFieldSelectAllOnTab(unitsSize);
+	    EDialog.makeTextFieldSelectAllOnTab(xOffset);
+	    EDialog.makeTextFieldSelectAllOnTab(yOffset);
+	    EDialog.makeTextFieldSelectAllOnTab(boxedHeight);
+	    EDialog.makeTextFieldSelectAllOnTab(boxedWidth);
+
         // pure "parameters" dialog doesn't have "Highlight Owner" button
         if (paramDialog) remove(seeNode);
 
@@ -195,13 +204,13 @@ public class TextInfoPanel extends javax.swing.JPanel
         });
     }
 
-    public void setOffsets(Point2D offset)
-    {
-        loading = true;
-        xOffset.setText(TextUtils.formatDouble(offset.getX()));
-        yOffset.setText(TextUtils.formatDouble(offset.getY()));
-        loading = false;
-    }
+//    public void setOffsets(Point2D offset)
+//    {
+//        loading = true;
+//        xOffset.setText(TextUtils.formatDistance(offset.getX()));
+//        yOffset.setText(TextUtils.formatDistance(offset.getY()));
+//        loading = false;
+//    }
 
     /**
      * Set what the dialog displays: It can display and allow editing of the settings
@@ -295,8 +304,10 @@ public class TextInfoPanel extends javax.swing.JPanel
         		initialYOffset = off.getY();
         	}
         }
-        xOffset.setText(TextUtils.formatDouble(initialXOffset));
-        yOffset.setText(TextUtils.formatDouble(initialYOffset));
+        Cell cell = WindowFrame.getCurrentCell();
+        Technology tech = (cell == null ? null : cell.getTechnology());
+        xOffset.setText(TextUtils.formatDistance(initialXOffset, tech));
+        yOffset.setText(TextUtils.formatDistance(initialYOffset, tech));
 
         // set the "invisible outside cell"
         initialInvisibleOutsideCell = td.isInterior();
@@ -405,24 +416,22 @@ public class TextInfoPanel extends javax.swing.JPanel
         // set the color
         initialColorIndex = td.getColorIndex();
         int [] colorIndices = EGraphics.getColorIndices();
-        int colorComboIndex = Arrays.binarySearch(colorIndices, initialColorIndex);
-//      int colorComboIndex = 0;
-//		for(int i=0; i<colorIndices.length; i++)
-//        {
-//			if (colorIndices[i] == initialColorIndex)
-//            {
-//                colorComboIndex = i+1;
-//                break; // No need of checking the rest of the array
-//            }
-//        }
-//        if (newValue != -1) newValue += 1;
-//        if (newValue != colorComboIndex)
-//            System.out.println("Wrong calculation");
+//        int colorComboIndex = Arrays.binarySearch(colorIndices, initialColorIndex);  // Arrays.binarySearch doesn't work on non-sorted array
+      int colorComboIndex = -1;
+		for(int i=0; i<colorIndices.length; i++)
+        {
+			if (colorIndices[i] == initialColorIndex)
+            {
+                colorComboIndex = i; // +1;
+                break; // No need of checking the rest of the array
+            }
+        }
         colorComboIndex = (colorComboIndex != -1) ? colorComboIndex + 1 : 0;
         textColorComboBox.setSelectedIndex(colorComboIndex);
 
         // report the global text scale
-        globalTextScale.setText("Scaled by " + TextUtils.formatDouble(User.getGlobalTextScale()*100) + "%");
+        double textScale = wnd != null ? wnd.getGlobalTextScale() : User.getGlobalTextScale();
+        globalTextScale.setText("Scaled by " + TextUtils.formatDouble(textScale*100) + "%");
 
         loading = false;
     }
@@ -469,8 +478,10 @@ public class TextInfoPanel extends javax.swing.JPanel
         }
 
         // handle changes to the offset
-        double xd = TextUtils.atof(xOffset.getText());
-        double yd = TextUtils.atof(yOffset.getText());
+        Cell cell = WindowFrame.getCurrentCell();
+        Technology tech = (cell == null ? null : cell.getTechnology());
+        double xd = TextUtils.atofDistance(xOffset.getText(), tech);
+        double yd = TextUtils.atofDistance(yOffset.getText(), tech);
         td = td.withOff(xd, yd);
 
         // handle changes to the rotation
@@ -545,8 +556,10 @@ public class TextInfoPanel extends javax.swing.JPanel
         if (!newSize.equals(initialSize)) changed = true;
 
         // handle changes to the offset
-        double currentXOffset = TextUtils.atof(xOffset.getText());
-        double currentYOffset = TextUtils.atof(yOffset.getText());
+        Cell cell = WindowFrame.getCurrentCell();
+        Technology tech = (cell == null ? null : cell.getTechnology());
+        double currentXOffset = TextUtils.atofDistance(xOffset.getText(), tech);
+        double currentYOffset = TextUtils.atofDistance(yOffset.getText(), tech);
         if (!DBMath.doublesEqual(currentXOffset, initialXOffset) ||
                 !DBMath.doublesEqual(currentYOffset, initialYOffset))
             changed = true;
@@ -773,12 +786,7 @@ public class TextInfoPanel extends javax.swing.JPanel
             td.setInterior(invis);
 
             // handle changes to the font
-            if (font.equals("DEFAULT FONT")) td.setFace(0);
-            else {
-                TextDescriptor.ActiveFont newFont = TextDescriptor.ActiveFont.findActiveFont(font);
-                int newFontIndex = newFont != null ? newFont.getIndex() : 0;
-                td.setFace(newFontIndex);
-            }
+            td.setFaceWithActiveFont(font);
 
             // handle changes to italic / bold / underline
             td.setItalic(italic);
