@@ -23,7 +23,6 @@
  */
 package com.sun.electric.tool.user.dialogs.options;
 
-import com.sun.electric.database.geometry.GenMath;
 import com.sun.electric.database.text.TextUtils;
 import com.sun.electric.technology.Layer;
 import com.sun.electric.technology.Technology;
@@ -36,9 +35,7 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
-import java.util.HashMap;
 import java.util.Iterator;
-import java.util.Map;
 
 import javax.swing.DefaultListModel;
 import javax.swing.JList;
@@ -69,14 +66,16 @@ public class CoverageTab extends PreferencePanel
 	    EDialog.makeTextFieldSelectAllOnTab(deltaYField);
 	}
 
-	/** return the panel to use for this preferences tab. */
-	public JPanel getPanel() { return layerCoverage; }
+	/** return the panel to use for the user preferences. */
+    @Override
+	public JPanel getUserPreferencesPanel() { return layerCoverage; }
 
 	/** return the name of this preferences tab. */
+    @Override
 	public String getName() { return "Coverage"; }
 
 	private boolean layerDataChanging = false;
-	private Map<Layer,GenMath.MutableDouble> layerAreaMap;
+    private LayerCoverageTool.LayerCoveragePreferences lcp;
 	private DefaultListModel layerListModel;
 	private JList layerJList;
 
@@ -84,9 +83,10 @@ public class CoverageTab extends PreferencePanel
 	 * Method called at the start of the dialog.
 	 * Caches current values and displays them in the New Arcs tab.
 	 */
+    @Override
 	public void init()
 	{
-		layerAreaMap = new HashMap<Layer,GenMath.MutableDouble>();
+		lcp = new LayerCoverageTool.LayerCoveragePreferences(false);
 		layerListModel = new DefaultListModel();
 		layerJList = new JList(layerListModel);
 		layerJList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
@@ -96,17 +96,10 @@ public class CoverageTab extends PreferencePanel
 		{
 			Technology tech = tIt.next();
 			technologySelection.addItem(tech.getTechName());
-
-			for(Iterator<Layer> it = tech.getLayers(); it.hasNext(); )
-			{
-				Layer layer = it.next();
-				double val = layer.getAreaCoverage();
-				layerAreaMap.put(layer, new GenMath.MutableDouble(val));
-			}
 		}
 		layerDataChanging = false;
 		layerAreaField.getDocument().addDocumentListener(new CoverageDocumentListener(this));
-		
+
 		technologySelection.addActionListener(new ActionListener()
 		{
 			public void actionPerformed(ActionEvent evt) { newTechSelected(); }
@@ -128,15 +121,14 @@ public class CoverageTab extends PreferencePanel
 		for(Iterator<Layer> it = tech.getLayers(); it.hasNext(); )
 		{
 			Layer layer = it.next();
-			GenMath.MutableDouble val = layerAreaMap.get(layer);
-			if (val == null) continue;
-			layerListModel.addElement(getLineString(layer, val.doubleValue()));
+			layerListModel.addElement(getLineString(layer, lcp.getAreaCoverage(layer)));
 		}
 		layerJList.setSelectedIndex(0);
-		widthField.setText(Double.toString(LayerCoverageTool.getWidth(tech)));
-		heightField.setText(Double.toString(LayerCoverageTool.getHeight(tech)));
-		deltaXField.setText(Double.toString(LayerCoverageTool.getDeltaX(tech)));
-		deltaYField.setText(Double.toString(LayerCoverageTool.getDeltaY(tech)));
+        double techScale = tech.getScale();
+		widthField.setText(TextUtils.formatDistance(lcp.widthInMicrons/techScale));
+		heightField.setText(TextUtils.formatDistance(lcp.heightInMicrons/techScale));
+		deltaXField.setText(TextUtils.formatDistance(lcp.deltaXInMicrons/techScale));
+		deltaYField.setText(TextUtils.formatDistance(lcp.deltaYInMicrons/techScale));
 		layerDataChanging = false;
 
 		layerValueChanged(false);
@@ -188,15 +180,12 @@ public class CoverageTab extends PreferencePanel
 		String techName = (String)technologySelection.getSelectedItem();
 		Technology tech = Technology.findTechnology(techName);
 		if (tech == null) return;
-		
+
 		String primName = (String)layerJList.getSelectedValue();
 		int spacePos = primName.indexOf(' ');
 		if (spacePos >= 0) primName = primName.substring(0, spacePos);
 		Layer layer = tech.findLayer(primName);
-		Object obj = layerAreaMap.get(layer);
-		if (obj == null) return;  // it should not happen though
-		GenMath.MutableDouble value = (GenMath.MutableDouble)obj;
-		double origValue = value.doubleValue();
+		double origValue = lcp.getAreaCoverage(layer);
 		if (set)
 		{
 			double val = 0;
@@ -221,18 +210,19 @@ public class CoverageTab extends PreferencePanel
 				SwingUtilities.invokeLater(new SetOriginalValue(layerAreaField, origValue));
 				return;
 			}
-			value.setValue(val);
+            lcp.setAreaCoverageInfo(layer, val);
 			int lineNo = layerJList.getSelectedIndex();
 			layerListModel.setElementAt(getLineString(layer, val), lineNo);
 		}
 		else
-			layerAreaField.setText(TextUtils.formatDouble(value.doubleValue()));
+			layerAreaField.setText(TextUtils.formatDouble(origValue));
 	}
 
 	/**
 	 * Method called when the "OK" panel is hit.
 	 * Updates any changed fields in the New Arcs tab.
 	 */
+    @Override
 	public void term()
 	{
 		// Default values are 50mm x 50 mm
@@ -240,53 +230,26 @@ public class CoverageTab extends PreferencePanel
 		Technology tech = Technology.findTechnology(techName);
 		if (tech != null)
 		{
-			double val = TextUtils.atof(widthField.getText());
-			if (val != LayerCoverageTool.getWidth(tech)) LayerCoverageTool.setWidth(val, tech);
-			val = TextUtils.atof(heightField.getText());
-			if (val != LayerCoverageTool.getHeight(tech)) LayerCoverageTool.setHeight(val, tech);
-			val = TextUtils.atof(deltaXField.getText());
-			if (val != LayerCoverageTool.getDeltaX(tech)) LayerCoverageTool.setDeltaX(val, tech);
-			val = TextUtils.atof(deltaYField.getText());
-			if (val != LayerCoverageTool.getDeltaY(tech)) LayerCoverageTool.setDeltaY(val, tech);
+            double techScale = tech.getScale();
+            lcp.widthInMicrons = TextUtils.atofDistance(widthField.getText())*techScale;
+            lcp.heightInMicrons = TextUtils.atofDistance(heightField.getText())*techScale;
+            lcp.deltaXInMicrons = TextUtils.atofDistance(deltaXField.getText())*techScale;
+            lcp.deltaYInMicrons = TextUtils.atofDistance(deltaYField.getText())*techScale;
 		}
 
-		for(Iterator<Technology> tIt = Technology.getTechnologies(); tIt.hasNext(); )
-		{
-			tech = tIt.next();
-			for(Iterator<Layer> it = tech.getLayers(); it.hasNext(); )
-			{
-				Layer layer = it.next();
-				Object obj = layerAreaMap.get(layer);
-				if (obj == null) continue;  // it should not happen though
-				GenMath.MutableDouble value = (GenMath.MutableDouble)obj;
-				layer.setAreaCoverageInfo(value.doubleValue());
-			}
-		}
+        putPrefs(lcp);
 	}
 
 	/**
 	 * Method called when the factory reset is requested.
 	 */
+    @Override
 	public void reset()
 	{
-		for(Iterator<Technology> tIt = Technology.getTechnologies(); tIt.hasNext(); )
-		{
-			Technology tech = tIt.next();
-			for(Iterator<Layer> it = tech.getLayers(); it.hasNext(); )
-			{
-				Layer layer = it.next();
-				if (layer.getFactoryAreaCoverage() != layer.getAreaCoverage())
-					layer.setAreaCoverageInfo(layer.getFactoryAreaCoverage());
-			}
-			if (LayerCoverageTool.getFactoryWidth(tech) != LayerCoverageTool.getWidth(tech))
-				LayerCoverageTool.setWidth(LayerCoverageTool.getFactoryWidth(tech), tech);
-			if (LayerCoverageTool.getFactoryHeight(tech) != LayerCoverageTool.getHeight(tech))
-				LayerCoverageTool.setHeight(LayerCoverageTool.getFactoryHeight(tech), tech);
-			if (LayerCoverageTool.getFactoryDeltaX(tech) != LayerCoverageTool.getDeltaX(tech))
-				LayerCoverageTool.setDeltaX(LayerCoverageTool.getFactoryDeltaX(tech), tech);
-			if (LayerCoverageTool.getFactoryDeltaY(tech) != LayerCoverageTool.getDeltaY(tech))
-				LayerCoverageTool.setDeltaY(LayerCoverageTool.getFactoryDeltaY(tech), tech);
-		}
+        if (lcp == null)
+            return; // nothing to reset
+        lcp.reset();
+        putPrefs(lcp);
 	}
 
 	/** This method is called from within the constructor to
