@@ -171,7 +171,7 @@ public class CellMenu {
 
 			SEPARATOR,
 
-			// mnemonic keys available:  BC      JKLM OPQRS   WXYZ
+			// mnemonic keys available:  BC      JK M OPQRS   WXYZ
 			new EMenu("Cell In_fo",
 				new EMenuItem("_Describe this Cell") { public void run() {
 					CellLists.describeThisCellCommand(); }},
@@ -189,10 +189,12 @@ public class CellMenu {
 				new EMenuItem("Number of _Transistors") { public void run() {
 					CellLists.numberOfTransistorsCommand(); }},
 				SEPARATOR,
-				new EMenuItem("Graphically, _Entire Library") { public void run() {
+				new EMenuItem("Cell Graph, _Entire Library") { public void run() {
 					CircuitChanges.graphCellsInLibrary(); }},
-				new EMenuItem("Graphically, _From Current Cell") { public void run() {
-					CircuitChanges.graphCellsFromCell(); }}),
+				new EMenuItem("Cell Graph, _From Current Cell") { public void run() {
+					CircuitChanges.graphCellsFromCell(); }},
+				new EMenuItem("_Library Graph") { public void run() {
+					CircuitChanges.graphLibraries(); }}),
 
 			new EMenuItem("Cell Propertie_s...") { public void run() {
 				cellControlCommand(); }},
@@ -489,7 +491,7 @@ public class CellMenu {
 		Point2D mouseDB = curEdit.screenToDatabase((int)mouse.getX(), (int)mouse.getY());
 		EPoint point = new EPoint(mouseDB.getX(), mouseDB.getY());
 		GeometrySearch.GeometrySearchResult foundAtTopLevel = null;
-		GeometrySearch search = new GeometrySearch();
+		GeometrySearch search = new GeometrySearch(curEdit.getLayerVisibility());
 		List<GeometrySearch.GeometrySearchResult> possibleTargets = search.searchGeometries(cell, point, true);
 
 		// eliminate results at the top level and duplicate results at any lower level
@@ -704,7 +706,7 @@ public class CellMenu {
 					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, new String[] {"Yes", "Cancel"}, "Yes");
 				if (response != 0) return;
 			}
-			new CellChangeJobs.DuplicateCell(cell, newName, allInGroup);
+			new CellChangeJobs.DuplicateCell(cell, newName, cell.getLibrary(), allInGroup, false);
 		}
 	}
 
