@@ -58,7 +58,7 @@ import com.sun.electric.tool.user.ActivityLogger;
 import com.sun.electric.tool.user.CircuitChangeJobs;
 import com.sun.electric.tool.user.CircuitChanges;
 import com.sun.electric.tool.user.Clipboard;
-import com.sun.electric.tool.user.Highlight2;
+import com.sun.electric.tool.user.Highlight;
 import com.sun.electric.tool.user.Highlighter;
 import com.sun.electric.tool.user.User;
 import com.sun.electric.tool.user.dialogs.Array;
@@ -89,6 +89,7 @@ import com.sun.electric.tool.user.ui.CurveListener;
 import com.sun.electric.tool.user.ui.EditWindow;
 import com.sun.electric.tool.user.ui.ErrorLoggerTree;
 import com.sun.electric.tool.user.ui.LayerVisibility;
+import com.sun.electric.tool.user.ui.MessagesWindow;
 import com.sun.electric.tool.user.ui.OutlineListener;
 import com.sun.electric.tool.user.ui.PaletteFrame;
 import com.sun.electric.tool.user.ui.SizeListener;
@@ -237,7 +238,11 @@ public class EditMenu {
 				new EMenuItem("Align Vertically to _Bottom") { public void run() {
 					CircuitChanges.alignNodes(false, 1); }},
 				new EMenuItem("Align _Vertically to Center") { public void run() {
-					CircuitChanges.alignNodes(false, 2); }}),
+					CircuitChanges.alignNodes(false, 2); }},
+                SEPARATOR,
+				new EMenuItem("Cell Center to Center of Selection") { public void run() {
+					CircuitChanges.cellCenterToCenterOfSelection(); }}
+                      ),
 
 			SEPARATOR,
 
@@ -429,7 +434,7 @@ public class EditMenu {
 				new EMenuItem("Technology Creation _Wizard...") { public void run() {
 					TechEditWizard.techEditWizardCommand(); }}),
 
-		// mnemonic keys available:  B   F    K M   Q        Z
+		// mnemonic keys available:      F    K     Q        Z
 			new EMenu("_Selection",
 				new EMenuItem("Sele_ct All", 'A') { public void run() {
 					selectAllCommand(); }},
@@ -447,7 +452,7 @@ public class EditMenu {
 				new EMenuItem("Select _Previous Like This") { public void run() {
 					selectNextLikeThisCommand(false); }},
 				SEPARATOR,
-				new EMenuItem("_Select Object...") { public void run() {
+				new EMenuItem("Select O_bject...") { public void run() {
 					SelectObject.selectObjectDialog(null, false); }},
 				new EMenuItem("Deselect All _Arcs") { public void run() {
 					deselectAllArcsCommand(); }},
@@ -468,9 +473,13 @@ public class EditMenu {
 					selectEnclosedObjectsCommand(); }},
 				SEPARATOR,
 				new EMenuItem("Show Ne_xt Error", KeyStroke.getKeyStroke('>')) { public void run() {
-					showNextErrorCommand(); }},
+					showNextErrorCommand(true); }},
 				new EMenuItem("Show Pre_vious Error", KeyStroke.getKeyStroke('<')) { public void run() {
-					showPrevErrorCommand(); }},
+					showPrevErrorCommand(true); }},
+				new EMenuItem("Show Next Error, _same Window", KeyStroke.getKeyStroke(']')) { public void run() {
+					showNextErrorCommand(false); }},
+				new EMenuItem("Show Previous Error, sa_me Window", KeyStroke.getKeyStroke('[')) { public void run() {
+					showPrevErrorCommand(false); }},
                 new EMenuItem("Show Single Geometry", KeyStroke.getKeyStroke(KeyEvent.VK_SLASH, 0)) { public void run() {
 					showSingleGeometryCommand(); }},
                 new EMenuItem("Show Current Co_llection of Errors") { public void run() {
@@ -508,7 +517,7 @@ public class EditMenu {
 		} else
 		{
 			int [] counts = new int[5];
-			NodeInst theNode = Highlight2.getInfoCommand(wnd.getHighlighter().getHighlights(), counts);
+			NodeInst theNode = Highlight.getInfoCommand(wnd.getHighlighter().getHighlights(), counts);
 			// information about the selected items
 			int arcCount = counts[0];
 			int nodeCount = counts[1];
@@ -902,7 +911,15 @@ public class EditMenu {
 	 */
 	public static void selectAllCommand()
 	{
-		doSelection(false, false);
+        // is this the messages window?
+        MessagesWindow mw = TopLevel.getMessagesWindow();
+        if (mw.isFocusOwner())
+        {
+        	mw.selectAll();
+        	return;
+        }
+
+        doSelection(false, false);
 	}
 
 	/**
@@ -1054,7 +1071,7 @@ public class EditMenu {
 
 		// make a set of prototypes and characteristics to match
 		Set<Object> likeThis = new HashSet<Object>();
-		for(Highlight2 h : highlighter.getHighlights())
+		for(Highlight h : highlighter.getHighlights())
 		{
 			// handle attribute text
 			if (h.isHighlightText())
@@ -1157,7 +1174,7 @@ public class EditMenu {
 		EditWindow wnd = EditWindow.getCurrent();
 		if (wnd == null) return;
 		Highlighter highlighter = wnd.getHighlighter();
-		Highlight2 high = highlighter.getOneHighlight();
+		Highlight high = highlighter.getOneHighlight();
 		if (high == null) return;
 		ElectricObject eObj = high.getElectricObject();
 		if (high.isHighlightEOBJ())
@@ -1323,8 +1340,8 @@ public class EditMenu {
         if (wnd == null) return;
         Highlighter highlighter = wnd.getHighlighter();
 
-        List<Highlight2> newHighList = new ArrayList<Highlight2>();
-        for(Highlight2 h : highlighter.getHighlights())
+        List<Highlight> newHighList = new ArrayList<Highlight>();
+        for(Highlight h : highlighter.getHighlights())
         {
             if (h.isHighlightEOBJ() || h.isHighlightText())
             {
@@ -1425,10 +1442,11 @@ public class EditMenu {
     /**
      * This method implements the command to show the next logged error.
      * The error log lists the results of the latest command (DRC, NCC, etc.)
+     * @param separateWindow true to show each cell in its own window; false to show in the current window.
      */
-    private static void showNextErrorCommand()
+    private static void showNextErrorCommand(boolean separateWindow)
     {
-        String msg = ErrorLoggerTree.reportNextMessage();
+        String msg = ErrorLoggerTree.reportNextMessage(separateWindow);
         System.out.println(msg);
     }
 
@@ -1439,16 +1457,17 @@ public class EditMenu {
      */
     private static void showSingleGeometryCommand()
     {
-        ErrorLoggerTree.reportSingleGeometry();
+        ErrorLoggerTree.reportSingleGeometry(true);
     }
 
     /**
      * This method implements the command to show the last logged error.
      * The error log lists the results of the latest command (DRC, NCC, etc.)
+     * @param separateWindow true to show each cell in its own window; false to show in the current window.
      */
-    private static void showPrevErrorCommand()
+    private static void showPrevErrorCommand(boolean separateWindow)
     {
-        String msg = ErrorLoggerTree.reportPrevMessage();
+        String msg = ErrorLoggerTree.reportPrevMessage(separateWindow);
         System.out.println(msg);
     }
 
@@ -1741,8 +1760,8 @@ public class EditMenu {
             if (np.isNotUsed()) continue;
             PrimitiveNode.Function fun = np.getFunction();
             totalCount++;
-            if (fun == PrimitiveNode.Function.PIN) pinCount++; else
-            if (fun == PrimitiveNode.Function.CONTACT || fun == PrimitiveNode.Function.CONNECT) contactCount++; else
+            if (fun.isPin()) pinCount++; else
+            if (fun.isContact() || fun == PrimitiveNode.Function.CONNECT) contactCount++; else
             if (fun == PrimitiveNode.Function.NODE) pureCount++;
         }
         if (pinCount > 0)
@@ -1755,7 +1774,7 @@ public class EditMenu {
                 PrimitiveNode np = it.next();
                 if (np.isNotUsed()) continue;
                 PrimitiveNode.Function fun = np.getFunction();
-                if (fun != PrimitiveNode.Function.PIN) continue;
+                if (!fun.isPin()) continue;
                 String addThis = " " + np.getName();
                 if (sb.length() + addThis.length() > pageWidth)
                 {
@@ -1777,7 +1796,7 @@ public class EditMenu {
                 PrimitiveNode np = it.next();
                 if (np.isNotUsed()) continue;
                 PrimitiveNode.Function fun = np.getFunction();
-                if (fun != PrimitiveNode.Function.CONTACT && fun != PrimitiveNode.Function.CONNECT)
+                if (!fun.isContact() && fun != PrimitiveNode.Function.CONNECT)
                     continue;
                 String addThis = " " + np.getName();
                 if (sb.length() + addThis.length() > pageWidth)
@@ -1800,7 +1819,7 @@ public class EditMenu {
                 PrimitiveNode np = it.next();
                 if (np.isNotUsed()) continue;
                 PrimitiveNode.Function fun = np.getFunction();
-                if (fun == PrimitiveNode.Function.PIN || fun == PrimitiveNode.Function.CONTACT ||
+                if (fun.isPin() || fun.isContact() ||
                     fun == PrimitiveNode.Function.CONNECT || fun == PrimitiveNode.Function.NODE)
                     continue;
                 String addThis = " " + np.getName();
