@@ -34,6 +34,7 @@ import com.sun.electric.database.hierarchy.Cell;
 import com.sun.electric.database.hierarchy.EDatabase;
 import com.sun.electric.database.hierarchy.Library;
 import com.sun.electric.database.hierarchy.View;
+import com.sun.electric.database.id.CellId;
 import com.sun.electric.database.id.LibId;
 import com.sun.electric.database.text.Pref;
 import com.sun.electric.database.text.Setting;
@@ -44,7 +45,6 @@ import com.sun.electric.technology.Technology;
 import com.sun.electric.tool.Client;
 import com.sun.electric.tool.Job;
 import com.sun.electric.tool.JobException;
-import com.sun.electric.tool.JobManager;
 import com.sun.electric.tool.cvspm.CVS;
 import com.sun.electric.tool.cvspm.CVSLibrary;
 import com.sun.electric.tool.cvspm.Commit;
@@ -104,6 +104,7 @@ import java.io.File;
 import java.io.Serializable;
 import java.net.URL;
 import java.util.ArrayList;
+import java.util.BitSet;
 import java.util.Collection;
 import java.util.Date;
 import java.util.HashMap;
@@ -130,10 +131,11 @@ import javax.swing.SwingUtilities;
  */
 public class FileMenu {
 
-    public static final EMenu openRecentLibs = new EMenu("Open Recent Library", new ArrayList<EMenuItem>());
+    private static EMenu openRecentLibs;
     public static final String openJobName = "Read External Library";
 
     static EMenu makeMenu() {
+        openRecentLibs = new EMenu("Open Recent Library", new ArrayList<EMenuItem>());
 		/****************************** THE FILE MENU ******************************/
 
 		// mnemonic keys available:    D               T  WXYZ
@@ -419,7 +421,7 @@ public class FileMenu {
         FileType defType = getLibraryFormat(fileName, null);
         if (defType == null) {
             // no valid extension, search for file with extension
-            URL f = TextUtils.makeURLToFile(fileName + "." + FileType.JELIB.getExtensions()[0]);
+            URL f = TextUtils.makeURLToFile(fileName + "." + FileType.JELIB.getFirstExtension());
             if (TextUtils.URLExists(f, null)) {
                 defType = FileType.JELIB;
                 file = f;
@@ -427,7 +429,7 @@ public class FileMenu {
         }
         if (defType == null) {
             // no valid extension, search for file with extension
-            URL f = TextUtils.makeURLToFile(fileName + "." + FileType.ELIB.getExtensions()[0]);
+            URL f = TextUtils.makeURLToFile(fileName + "." + FileType.ELIB.getFirstExtension());
             if (TextUtils.URLExists(f, null)) {
                 defType = FileType.ELIB;
                 file = f;
@@ -435,7 +437,7 @@ public class FileMenu {
         }
         if (defType == null) {
             // no valid extension, search for file with extension
-            URL f = TextUtils.makeURLToFile(fileName + "." + FileType.DELIB.getExtensions()[0]);
+            URL f = TextUtils.makeURLToFile(fileName + "." + FileType.DELIB.getFirstExtension());
             if (TextUtils.URLExists(f, null)) {
                 defType = FileType.DELIB;
                 file = f;
@@ -443,7 +445,7 @@ public class FileMenu {
         }
         if (defType == null) {
             // no valid extension, search for file with extension
-            URL f = TextUtils.makeURLToFile(fileName + "." + FileType.READABLEDUMP.getExtensions()[0]);
+            URL f = TextUtils.makeURLToFile(fileName + "." + FileType.READABLEDUMP.getFirstExtension());
             if (TextUtils.URLExists(f, null)) {
                 defType = FileType.READABLEDUMP;
                 file = f;
@@ -461,17 +463,7 @@ public class FileMenu {
     /** Get the type from the fileName, or if no valid Library type found, return defaultType.
      */
     public static FileType getLibraryFormat(String fileName, FileType defaultType) {
-        if (fileName != null)
-        {
-            if (fileName.endsWith(File.separator)) {
-                fileName = fileName.substring(0, fileName.length()-File.separator.length());
-            }
-            for (FileType type :  FileType.libraryTypes)
-            {
-                if (fileName.endsWith("."+type.getExtensions()[0])) return type;
-            }
-        }
-        return defaultType;
+        return FileType.getLibraryFormat(fileName, defaultType);
     }
 
     private static class ReadProjectSettingsFromLibrary extends Job {
@@ -619,6 +611,12 @@ public class FileMenu {
             Cell showThisCell = cellName != null ? lib.findNodeProto(cellName) : lib.getCurCell();
         	doneOpeningLibrary(showThisCell);
             convertVarsToModelFiles(newLibs);
+            for (Library lib: newLibs) {
+                for (Iterator<Cell> it = lib.getCells(); it.hasNext(); ) {
+                    Cell cell = it.next();
+                    cell.loadExpandStatus();
+                }
+            }
             if (CVS.isEnabled()) {
                 if (saveTask != null)
                     saveTask.librarySaved();
@@ -691,6 +689,7 @@ public class FileMenu {
 		private Library createLib;
 		private Technology tech;
         private Map<Library,Cell> currentCells = new HashMap<Library,Cell>();
+        private Map<CellId,BitSet> nodesToExpand = new HashMap<CellId,BitSet>();
 
 		public ImportLibrary(URL fileURL, FileType type, Library libToRead,
             Library deleteLib, Technology tech, RenameAndSaveLibraryTask saveTask)
@@ -711,7 +710,8 @@ public class FileMenu {
             }
 
 			prefs = Input.getInputPreferences(type, false);
-            prefs.initFromUserDefaults();
+            if (prefs != null)
+                prefs.initFromUserDefaults();
 			startJob();
 		}
 
@@ -731,7 +731,7 @@ public class FileMenu {
 				if (!deleteLib.kill("replace")) return false;
 				deleteLib = null;
 			}
-			createLib = Input.importLibrary(prefs, fileURL, type, curLib, tech, currentCells, this);
+			createLib = Input.importLibrary(prefs, fileURL, type, curLib, tech, currentCells, nodesToExpand, this);
 			if (createLib == null) return false;
 
             // new library open: check for default "noname" library and close if empty
@@ -744,6 +744,7 @@ public class FileMenu {
 
 			fieldVariableChanged("createLib");
             fieldVariableChanged("currentCells");
+            fieldVariableChanged("nodesToExpand");
 			return true;
 		}
 
@@ -761,6 +762,7 @@ public class FileMenu {
                 lib.setCurCell(cell);
             }
         	Cell showThisCell = createLib.getCurCell();
+            getDatabase().expandNodes(nodesToExpand);
         	doneOpeningLibrary(showThisCell);
         }
 	}
@@ -820,8 +822,10 @@ public class FileMenu {
 	/**
 	 * This method implements the command to import a library.
 	 * It is interactive, and pops up a dialog box.
+     * @param type Type of file to import.
 	 * @param wholeDirectory true to import a directory instead of a file.
 	 * @param canMerge true to allow merging into an existing library.
+     * @param techSpecific true if a particular technology is needed for the import.
 	 */
 	public static void importLibraryCommand(FileType type, boolean wholeDirectory, boolean canMerge, boolean techSpecific)
 	{
@@ -833,63 +837,73 @@ public class FileMenu {
 		{
 			fileName = OpenFile.chooseInputFile(type, null);
 		}
-		if (fileName != null)
-		{
-			URL fileURL = TextUtils.makeURLToFile(fileName);
-			String libName = TextUtils.getFileNameWithoutExtension(fileURL);
-			Library deleteLib = Library.findLibrary(libName);
-			Library libToRead = null;
-            RenameAndSaveLibraryTask saveTask = null;
-			if (deleteLib != null)
-			{
-				// library already exists, prompt for save
-				MutableBoolean wantToMerge = null;
-				if (canMerge) wantToMerge = new MutableBoolean(false);
-                Collection<RenameAndSaveLibraryTask> librariesToSave = preventLoss(deleteLib, 2, wantToMerge);
-                if (librariesToSave == null) return;
-                if (canMerge && wantToMerge.booleanValue())
+        if (fileName == null) return; // nothing to import
+        importLibraryCommandNoGUI(fileName, type, canMerge, techSpecific);
+    }
+
+    /**
+     * A non-interactive method to import a given file. Used in batch mode as well.
+     * @param fileName Name of the file to import
+     * @param type Type of the file to import.
+	 * @param canMerge true to allow merging into an existing library.
+     * @param techSpecific true if a particular technology is needed for the import.
+     */
+    public static void importLibraryCommandNoGUI(String fileName, FileType type, boolean canMerge, boolean techSpecific)
+    {
+        URL fileURL = TextUtils.makeURLToFile(fileName);
+        String libName = TextUtils.getFileNameWithoutExtension(fileURL);
+        Library deleteLib = Library.findLibrary(libName);
+        Library libToRead = null;
+        RenameAndSaveLibraryTask saveTask = null;
+        if (deleteLib != null)
+        {
+            // library already exists, prompt for save
+            MutableBoolean wantToMerge = null;
+            if (canMerge) wantToMerge = new MutableBoolean(false);
+            Collection<RenameAndSaveLibraryTask> librariesToSave = preventLoss(deleteLib, 2, wantToMerge);
+            if (librariesToSave == null) return;
+            if (canMerge && wantToMerge.booleanValue())
+            {
+                libToRead = deleteLib;
+                deleteLib = null;
+            } else
+            {
+                if (!librariesToSave.isEmpty())
                 {
-                	libToRead = deleteLib;
-                	deleteLib = null;
-                } else
-                {
-	                if (!librariesToSave.isEmpty())
-	                {
-	                    assert librariesToSave.size() == 1;
-	                    saveTask = librariesToSave.iterator().next();
-	                    assert saveTask.libId == deleteLib.getId();
-	                }
-					WindowFrame.removeLibraryReferences(deleteLib);
+                    assert librariesToSave.size() == 1;
+                    saveTask = librariesToSave.iterator().next();
+                    assert saveTask.libId == deleteLib.getId();
                 }
-			}
-			Technology tech = null;
-			if (techSpecific)
-			{
-				// prompt for the technology to use
-				List<Technology> layoutTechnologies = new ArrayList<Technology>();
-				String curTech = null;
-				for(Iterator<Technology> it = Technology.getTechnologies(); it.hasNext(); )
-				{
-					Technology t = it.next();
-					if (!t.isLayout()) continue;
-					if (t == Technology.getCurrent()) curTech = t.getTechName();
-					layoutTechnologies.add(t);
-				}
-				if (curTech == null) curTech = User.getSchematicTechnology().getTechName();
-				String [] techArray = new String[layoutTechnologies.size()];
-				for(int i=0; i<layoutTechnologies.size(); i++)
-					techArray[i] = layoutTechnologies.get(i).getTechName();
-				String chosen = (String)JOptionPane.showInputDialog(TopLevel.getCurrentJFrame(),
-					"Which layout technology should be used to import the file:", "Choose Technology",
-					JOptionPane.QUESTION_MESSAGE, null, techArray, curTech);
-				if (chosen == null) return;
-				tech = Technology.findTechnology(chosen);
-			}
-            if (type == FileType.ELIB || type == FileType.READABLEDUMP)
-                new ReadLibrary(fileURL, type, null, deleteLib, saveTask, null);
-            else
-                new ImportLibrary(fileURL, type, libToRead, deleteLib, tech, saveTask);
-		}
+                WindowFrame.removeLibraryReferences(deleteLib);
+            }
+        }
+        Technology tech = null;
+        if (techSpecific)
+        {
+            // prompt for the technology to use
+            List<Technology> layoutTechnologies = new ArrayList<Technology>();
+            String curTech = null;
+            for(Iterator<Technology> it = Technology.getTechnologies(); it.hasNext(); )
+            {
+                Technology t = it.next();
+                if (!t.isLayout()) continue;
+                if (t == Technology.getCurrent()) curTech = t.getTechName();
+                layoutTechnologies.add(t);
+            }
+            if (curTech == null) curTech = User.getSchematicTechnology().getTechName();
+            String [] techArray = new String[layoutTechnologies.size()];
+            for(int i=0; i<layoutTechnologies.size(); i++)
+                techArray[i] = layoutTechnologies.get(i).getTechName();
+            String chosen = (String)JOptionPane.showInputDialog(TopLevel.getCurrentJFrame(),
+                "Which layout technology should be used to import the file:", "Choose Technology",
+                JOptionPane.QUESTION_MESSAGE, null, techArray, curTech);
+            if (chosen == null) return;
+            tech = Technology.findTechnology(chosen);
+        }
+        if (type == FileType.ELIB || type == FileType.READABLEDUMP)
+            new ReadLibrary(fileURL, type, null, deleteLib, saveTask, null);
+        else
+            new ImportLibrary(fileURL, type, libToRead, deleteLib, tech, saveTask);
 	}
 
     public static void closeLibraryCommand(Library lib)
@@ -1045,8 +1059,8 @@ public class FileMenu {
             if (val == 1) return null;
         }
 
-        String [] extensions = type.getExtensions();
-        String extension = extensions[0];
+//        String [] extensions = type.getFirstExtension();
+        String extension = type.getFirstExtension();
         String fileName = null;
         if (!saveAs && lib.isFromDisk())
         {
@@ -1165,11 +1179,18 @@ public class FileMenu {
                 libFile = lib.getLibFile();
                 deletedCellFiles = new ArrayList<String>();
                 writtenCellFiles = new ArrayList<String>();
-                success = !Output.writeLibrary(lib, type, compatibleWith6, false, false, backupScheme, deletedCellFiles, writtenCellFiles);
+                Output.writeLibrary(lib, type, compatibleWith6, false, false, backupScheme, deletedCellFiles, writtenCellFiles);
+                success = true;
             } catch (Exception e) {
-                e.printStackTrace(System.out);
-                throw new JobException("Exception caught when saving files: " +
-                        e.getMessage() + "Please check your disk libraries");
+                if (Job.isThreadSafe()) {
+                     e.printStackTrace(System.out);
+                    throw new JobException("Exception caught when saving files: " +
+                            e.getMessage() + "Please check your disk libraries");
+                } else {
+                   // throwing a JobException here causes the UI to freeze...SMR
+                    System.out.println("Error saving files.  Please check your disk libraries");
+                    return null;
+                }
             }
             if (!success)
                 throw new JobException("Error saving files.  Please check your disk libraries");
@@ -1188,6 +1209,11 @@ public class FileMenu {
      */
     public static void saveAsLibraryCommand(Library lib)
     {
+        if (lib == null)
+        {
+            System.out.println("No library to save");
+            return;
+        }
         saveLibraryCommand(lib, FileType.DEFAULTLIB, false, true, true);
         WindowFrame.wantToRedoTitleNames();
     }
@@ -1261,7 +1287,7 @@ public class FileMenu {
                     String fullName = lib.getLibFile().getFile();
                     //if (fullName.endsWith("spiceparts.txt")) continue; // ignore spiceparts library
                     // match ".<word><endline>"
-                    fullName = fullName.replaceAll("\\.\\w*?$", "."+outType.getExtensions()[0]);
+                    fullName = fullName.replaceAll("\\.\\w*?$", "."+outType.getFirstExtension());
                     lib.setLibFile(TextUtils.makeURLToFile(fullName));
                 }
                 lib.setChanged();
@@ -1358,12 +1384,22 @@ public class FileMenu {
         {
             if (cell.getView() == View.DOC)
             {
-                System.out.println("Document cells can't be exported as postscript.");
-                JOptionPane.showMessageDialog(TopLevel.getCurrentJFrame(), "Document cells can't be exported as postscript.\n" +
+                String msg = "Document cells can't be exported as postscript.\n";
+                System.out.print(msg);
+                JOptionPane.showMessageDialog(TopLevel.getCurrentJFrame(), msg +
                         "Try \"Export -> Text Cell Contents\"",
                     "Exporting PS", JOptionPane.ERROR_MESSAGE);
                 return;
             }
+            if (!(wnd instanceof EditWindow))
+            {
+                String msg = "This type of window can't be exported as postscript.\n";
+                System.out.print(msg);
+                JOptionPane.showMessageDialog(TopLevel.getCurrentJFrame(), msg +
+                        "Try \"Export -> PNG (Portable Network Graphcs)\"",
+                    "Exporting PS", JOptionPane.ERROR_MESSAGE);
+                return;
+            }
 
             if (IOTool.isPrintEncapsulated()) type = FileType.EPS;
 			if (wnd instanceof WaveformWindow)
@@ -1374,8 +1410,8 @@ public class FileMenu {
 			}
         }
 
-		String [] extensions = type.getExtensions();
-        String filePath = cell.getName() + "." + extensions[0];
+//		String [] extensions = type.getExtensions();
+        String filePath = cell.getName() + "." + type.getFirstExtension();
 
         // special case for spice
         if (type == FileType.SPICE &&
@@ -1420,23 +1456,22 @@ public class FileMenu {
     private static class ExportImage extends Job
 	{
     	private String filePath;
-		private WindowContent wnd;
+		private WindowContent wc;
+        ElectricPrinter ep;
 
-		public ExportImage(String description, WindowContent wnd, String filePath)
+		public ExportImage(String description, WindowContent wc, String filePath)
 		{
 			super("Export "+description+" ("+FileType.PNG+")", User.getUserTool(), Job.Type.CLIENT_EXAMINE, null, null, Job.Priority.USER);
-			this.wnd = wnd;
+			this.wc = wc;
 			this.filePath = filePath;
+			PrinterJob pj = PrinterJob.getPrinterJob();
+			this.ep = getOutputPreferences(wc, pj);
 			startJob();
 		}
 		public boolean doIt() throws JobException
         {
-			PrinterJob pj = PrinterJob.getPrinterJob();
-	        ElectricPrinter ep = getOutputPreferences(wnd, pj);
             // Export has to be done in same thread as offscreen raster (valid for 3D at least)
-            wnd.writeImage(ep, filePath);
-//            BufferedImage img = wnd.getOffScreenImage(ep);
-//			PNG.writeImage(img, filePath);
+			wc.writeImage(ep, filePath);
             return true;
         }
 	}
@@ -1520,7 +1555,16 @@ public class FileMenu {
 			RepaintManager currentManager = RepaintManager.currentManager(overall);
 			currentManager.setDoubleBufferingEnabled(false);
 
-		    ElectricPrinter ep = getOutputPreferences(wf.getContent(), pj);
+            if (!(wf.getContent() instanceof EditWindow))
+            {
+                String message = "Can't print from a non-EditWindow.";
+                System.out.println(message);
+                JOptionPane.showMessageDialog(TopLevel.getCurrentJFrame(), message,
+                    "Printing Cell", JOptionPane.ERROR_MESSAGE);
+                return;
+            }
+
+            ElectricPrinter ep = getOutputPreferences(wf.getContent(), pj);
 			Dimension oldSize = overall.getSize();
 			ep.setOldSize(oldSize);
 
@@ -1671,7 +1715,7 @@ public class FileMenu {
             for (RenameAndSaveLibraryTask saveTask: saveTasks)
                 saveTask.librarySaved();
             try {
-                Library.saveExpandStatus();
+                getDatabase().saveExpandStatus();
                 Pref.getPrefRoot().flush();
             } catch (BackingStoreException e)
             {
@@ -1891,7 +1935,7 @@ public class FileMenu {
             return false;
         }
         // set libraries to save to panic dir
-        Snapshot panicSnapshot = JobManager.findValidSnapshot();
+        Snapshot panicSnapshot = Job.findValidSnapshot();
         boolean ok = !Output.writePanicSnapshot(panicSnapshot, panicDir, false);
         if (ok) {
             JOptionPane.showMessageDialog(TopLevel.getCurrentJFrame(), new String [] {"Libraries are saved to panic directory",
